#!/usr/bin/python

import argparse
import locale
import os
import re
import sys
import time
import warnings
from pytonik.cmd import lang
from pytonik import Version
from pytonik.cmd import server
from typing import Any, Callable, Dict, List, Pattern, Union
from pytonik.cmd.console import (  # type: ignore
    colorize, bold, green, red, turquoise, nocolor, color_terminal
)
import shutil
import zipfile

try:
    import readline

    if readline.__doc__ and 'libedit' in readline.__doc__:
        readline.parse_and_bind("bind ^I rl_complete")
        USE_LIBEDIT = True
    else:
        readline.parse_and_bind("tab: complete")
        USE_LIBEDIT = False
except ImportError:
    USE_LIBEDIT = False

PROMPT_PREFIX = '> '

if sys.platform == 'win32':
    # On Windows, show questions as bold because of color scheme of PowerShell
    # (refs: #5294).
    COLOR_QUESTION = 'bold'
else:
    COLOR_QUESTION = 'purple'


def pathwhich():
    pat = ""
    if sys.platform == 'win32':
        path = os.environ['PATH'].split(';')

        for p in path:
            if "Python" in p:
                # python.exe
                if 'python.exe' in p:
                    pat = p
                else:

                    if p[-1] == '\"' or p[-1] == '/':
                        pat = p[:-1] + '\"' + str("python.exe")
                        break
                    else:
                        pat = str(p) + "\'" + str("python.exe")
                        break

    else:
        try:
            pat = os.popen('which python').read()
        except Exception as arr:
            try:
                pat = os.environ['__PYVENV_LAUNCHER__']
            except Exception as arr:
                print(arr)
    return pat.replace("'", '')


# function to get input from terminal -- overridden by the test suite
def term_input(prompt: str) -> str:
    if sys.platform == 'win32':

        print(prompt, end='')
        return input('')
    else:
        return input(prompt)


def __(mes_id):
    try:
        userlang = locale.getlocale()[0]

        l_ = mes_id
        for l in lang.lang:
            if l == str(userlang):
                getla = lang.lang.get(l, '')

                if getla != "":
                    l_ = getla.get(mes_id, '')
                    if l_ != "":
                        l_ = getla.get(mes_id, '')
                    else:
                        l_ = mes_id
                else:
                    l_ = mes_id
    except Exception as arr:
        l_ = mes_id
    return str(l_)


class ValidationError(Exception):
    """Raised for validation errors."""


def is_path(x: str) -> str:
    x = os.path.expanduser(x)
    if not os.path.isdir(x):
        raise ValidationError(__("Please enter a valid path name."))
    return x


def boolean(x: str) -> bool:
    if x.upper() not in ('Y', 'YES', 'N', 'NO'):
        raise ValidationError(__("Please enter either 'y' or 'n'."))
    return x.upper() in ('Y', 'YES')


def allow_empty(x: str) -> str:
    return x


def nonempty(x: str) -> str:
    if not x:
        raise ValidationError(__("Please enter some text."))
    return x


def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] = nonempty) -> Union[str, bool]:  # NOQA
    while True:
        if default is not None:
            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
        else:
            prompt = PROMPT_PREFIX + text + ': '
        if USE_LIBEDIT:
            # Note: libedit has a problem for combination of ``input()`` and escape
            # sequence (see #5335).  To avoid the problem, all prompts are not colored
            # on libedit.
            pass
        else:
            prompt = colorize(COLOR_QUESTION, prompt, input_mode=True)
        x = term_input(prompt).strip()
        if default and not x:
            x = default
        try:
            x = validator(x)
        except ValidationError as err:
            print(red('* ' + str(err)))
            continue
        break
    return x


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        usage='%(prog)s [OPTIONS] <PROJECT_DIR>',
        epilog=__(
            "For more information, visit < https://pytonik.readthedocs.io >."),
        description=__("""Pytonik is a python framework built to enhance web development
        fast and easy, also help web developers to build more apps with less codes.
        it uses expressive architectural pattern, structured on model view controller MVC
        and bundles of component to reuse while deploying the framework."""))

    parser.add_argument('-q', '--quit', action='store_true', dest='quit',
                        default=None,
                        help=__('quit mode'))
    parser.add_argument('--version', action='version', dest='show_version',
                        version='%%(prog)s %s' % Version.VERSION_TEXT)

    parser.add_argument('path', metavar='PROJECT_DIR', default='.', nargs='?',
                        help=__('project root'))

    return parser


def ask(d: Dict) -> None:
    print(bold(__('Welcome to the Pytonik MVC Framework %s Start File Structure.' %
                  Version.VERSION_TEXT)))

    print(__('Please enter values for the following settings.'))

    if 'path' in d:
        print(bold(__('''Selected root path: %s''' % os.getcwd())))
        d['path'] = os.getcwd()

        d['step'] = do_prompt(__('Do you want to create file pytonik in this directory  (y/n)'),
                              'n', boolean)

    if d.get('step', '') is True:

        if 'project' not in d:

            print(__('''Provide project name, it will be use in project folder creation, avoid space while typing'''))

            d['project'] = do_prompt(bold('Project name'))
            if d.get('project', '') != "":

                if os.path.isdir(os.getcwd() + '/' + d.get('project', '')) is False:

                    print(bold(__('Project In-progress, please wait..')))

                    try:

                        os.mkdir(os.getcwd() + '/' +
                                 d.get('project', ''), mode=0o755)

                        print(
                            bold(__('Project {} Created Successfully'.format(d.get('project', '')))))

                    except Exception as err:
                        print(red('Unable to create project {} directory'.format(
                            os.getcwd() + '/' + d.get('project', ''))))

                    make_file(d)

                else:
                    msg = red(
                        __('Directory Already exist, overwrite ? (y/n)'.format(projectname=d.get('project', ''))))
                    d['cont'] = do_prompt(msg, 'n', boolean)
                    if d.get('cont') is True:

                        make_file(d)

                    else:
                        sys.exit(-1)

            else:
                d['quit'] = do_prompt(
                    __('Do you want to quit (y/n)'), 'n', boolean)

    else:
        d['quit'] = do_prompt(__('Do you want to quit (y/n)'), 'n', boolean)

    if d.get('quit', '') is True:
        sys.exit(-1)

    print()


def make_file(d):
    dst = os.getcwd() + '/' + d.get('project', '')
    try:

        src = os.path.dirname(os.path.abspath(__file__))

        zip_folder = src + '/land.zip'

        zip_folder = zip_folder.replace('\\', '/')

        with zipfile.ZipFile(zip_folder) as zf:
            zf.extractall(dst)

        print(bold(green(__('Project {} Is ready...'.format(d.get('project', ''))))))
    except Exception as err:
        try:

            src = os.path.dirname(os.path.abspath(__file__)) + "/land"

            for f in os.listdir(src):
                s = os.path.join(src, f)
                dt = os.path.join(dst, f)
                if os.path.isdir(s):
                    shutil.copytree(s, dt)
                else:
                    shutil.copy2(s, dt)

            print(bold(green(__('Project {} Is ready...'.format(d.get('project', ''))))))
        except Exception as err:
            print(err)

    # write / update python path
    dst_path = dst+"/public/index.py"
    dst_path_log = dst+"/app.log"
    if os.path.isfile(dst_path) == True:
        index_file = """{}
try:
\n  from pytonik import Web
\nexcept Exception as err:
\n  exit(err)
\nApp = Web.App()
\nApp.runs()
""".format("#!" + str(pathwhich()))
        file_open = open(dst_path, 'w+')
        file_open.write(index_file)
        try:
            os.chmod(dst_path, mode=0o755)
            os.chmod(dst_path_log, mode=0o777)
            print(bold(green(__('Permission Done...'))))

        except Exception as err:

            print(bold(red(__('Unable to Set permission '))))


def main(argv: List[str] = sys.argv[1:]) -> int:
    # parse options
    parser = get_parser()
    try:
        args = parser.parse_args(argv)

    except SystemExit as err:
        return err.code

    d = vars(args)
    ask(d)

    return ""


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import string
import os
from pytonik import Hash, Log, Version
log_msg = Log.Log()

try:
    import http.client as htp
    from http import client
    import urllib as urllib
except Exception as err:
    import httplib as htp
    import urllib2 as urllib


def limit_string(data="", length=10, readmore=""):
    if data != "":
        info = (data[:int(length)] + str(readmore)
                ) if len(data) > int(length) else data
    else:
        info = ""
    return info


def numberformat(num=int):

    if(int(num) > 1000000000):
        getcount = str(int(round((num/1000000000),)))+str('T')

    elif(num > 1000000):
        getcount = str(int(round((num/1000000), 1)))+str('M')

    elif(num > 1000):
        getcount = str(int(round((num/1000), 1)))+str('K')

    else:
        getcount = str(int(round((num/1), 1)))
    return getcount


def humanbytes(B):
    # Return the given bytes as a human friendly KB, MB, GB, or TB string'
    B = float(B)
    KB = float(1024)
    MB = float(KB ** 2)  # 1,048,576
    GB = float(KB ** 3)  # 1,073,741,824
    TB = float(KB ** 4)  # 1,099,511,627,776

    if B < KB:
        return '{0} {1}'.format(B, 'Bytes' if 0 == B > 1 else 'Byte')
    elif KB <= B < MB:
        return '{0:.2f} KB'.format(B/KB)
    elif MB <= B < GB:
        return '{0:.2f} MB'.format(B/MB)
    elif GB <= B < TB:
        return '{0:.2f} GB'.format(B/GB)
    elif TB <= B:
        return '{0:.2f} TB'.format(B/TB)


def file_get_contents(self, filename, use_include_path=0, context=None, offset=-1, maxlen=-1):
    if (filename.find('://') > 0):
        ret = urllib.urlopen(filename).read()
        if (offset > 0):
            ret = ret[offset:]
        if (maxlen > 0):
            ret = ret[:maxlen]
        return ret
    else:
        fp = open(filename, 'rb')
        try:
            if (offset > 0):
                fp.seek(offset)
            ret = fp.read(maxlen)
            return ret
        except Exception as err:
            log_msg.critical(err)
        finally:
            fp.close()


def creatdatetime(datetimeString, formate="%d-%m-%Y"):
    import datetime

    try:
        date = str(datetimeString).split(" ")[-1][:4]
        return datetime.datetime.strptime(date, formate)
        # return type(datetime.strptime(datetimeString, formate).date())
        # return datetimeString if datetimeString is "" or datetimeString is None  else datetime.strptime(datetimeString, formate).date()
    except Exception as err:
        return err


def datetime(formate="%Y-%m-%d %H:%M:%S"):
    from datetime import datetime

    try:
        return datetime.strftime(datetime.now(), formate)

    except Exception as err:
        return err


def iteri(dictionary="", itr="xid"):
    i = 0
    if dictionary != "" or dictionary != None:
        dist, apend = [], []
        for l in dictionary:
            i += 1
            ++i
            listv = l
            dist = {itr: i}
            dist.update(listv)
            apend.append(dist)
        return apend



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 17/11/2019.
import os as opy, re
from pytonik.util.Variable import Variable

class os(Variable):

    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):
        return None

    def __init__(self):
        if self.out('HTTP_USER_AGENT') != "":
            self.agent = self.out('HTTP_USER_AGENT')
        else:
            self.agent = None

        self.info = []
        self.name()
        self.name = self.info['OS']
        self.device = self.info['Device']

    def name(self):
         NAME = {
                "Windows": "PC",
				"Linux" : "PC",
				"Unix" : "PC",
				"Mac": "PC",
				"Android" : "Mobile",
				"Ubuntu" : "PC",
				"Chromium" : "PC",
				"iOS" : "PC",
				"DOS" : "PC",
				"JavaOS" : "PC",
				"Zorin" : "PC",
				"Elementary" : "PC",
				"NetWare" : "PC",
				"Papyros" : "PC",
				"Solaris" : "PC",
				"Symbian" : "Mobile",
				"Bharat" : "PC",
				"CentOS" : "PC",
				"ReactOS" : "PC",
                "iPhone" : "Mobile",
                "iPod" : "Mobile",
                "iPad" : "Mobile",
                "BlackBerry" : "Mobile",
                "Funtouch" : "Mobile",
                "LineageOS" : "Mobile",
                "BADA" : "Mobile",
                "Palm" : "Mobile",
                "Open" : "Mobile",
                "Maemo" : "Mobile",
                "Verdict" : "Mobile",

            }
         agent = str(self.agent)
         slipt1 = agent.split(")")
         slipt1 = str(slipt1).split("(")
         slipt1 = str(slipt1).split(";")
         slipt1 = str(slipt1).replace('"', " ").split('"')
         splits = str(slipt1).replace("'", " ").split(' ')

         for key in splits:
            if len(key) > 0:
                if NAME.get(key, None) is not None:
                    self.info = {'OS': key}
                    info = {'Device': NAME.get(key, "UNKNOWN")}
                    break
                else:
                    self.info = {'OS': "UNKNOWN"}
                    info = {'Device': "UNKNOWN"}


            else:
                self.info = {'OS': "UNKNOWN"}
                info = {'Device': "UNKNOWN"}

         self.info.update(info)





class browser(Variable):

    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):

        return None

    def __init__(self):
        if self.out('HTTP_USER_AGENT') !="":
            self.agent = self.out('HTTP_USER_AGENT')
        else:
            self.agent = None

        self.info = []
        self.name()
        self.name = self.info['Browser']
        self.version = self.info['Version']

    def name(self):
         NAME = {
                "Navigator": "Navigator",
                "Firefox": "Firefox",
                "Internet Explorer": "MSIE",
                "Chrome": "chrome",
                "Maxthon": "Maxthon",
                "Opera": "Opera",
                "OPR": "Opera",
                "Safari": "Safari",
                "Camino": "Camino",
                "SeaMonkey": "SeaMonkey",
                "K-Meleon": "K-Meleon",
                "Flock": "Flock",
                "Lunascape": "Lunascape",
                "Torch": "Torch",
                "Xtravo": "Xtravo",
                "Stainless": "Stainless",
                "Deepnet Explorer": "Deepnet",
                "QtWeb": "QtWeb",
                "CometBird": "CometBird",
                "xombrero": "xombrero",
                "Ultrabrowser": "Ultrabrowser",
                "Chromium": "Chromium",
                "Dooble": "Dooble"

            }
         agent = str(self.agent)
         slipt = agent.split(" ")
         for key in reversed(slipt):
             splits = key.split("/")

             if len(splits[0]) > 0 or len(splits[1]) > 0 :
                 if NAME.get(splits[0], None) is not None:
                     self.info = {'Browser': splits[0]}
                     info = {'Version': splits[1]}
                     break
                 else:
                     self.info = {'Browser': "UNKNOWN"}
                     info = {'Version': "UNKNOWN"}

             else:
                self.info = {'Browser': "UNKNOWN"}
                info = {'Version': "UNKNOWN"}


         self.info.update(info)


# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 08/01/2020.

class count:


    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):

        return None

    def __init__(self, *args,  **kwargs):

        return None


    def digit(self, num=0):
        num = float(num)
        if (float(num) > 1000000000) == True:
            getcount = str(float(round((num / 1000000000), 1)))[:-2] + str('T') if '.0' in str(float(round((num / 1000), 1))) else str(float(round((num / 1000000000), 1))) + str('T')


        elif (float(num) > 1000000):


            getcount = str(float(round((num / 1000000), 1)))[:-2] + str('M') if '.0' in str(float(round((num / 1000), 1))) else str(float(round((num / 1000000), 1))) + str('M')


        elif (float(num) > 1000) == True:

            getcount = str(float(round((num / 1000), 1)))[:-2]  + str('K')  if '.0' in str(float(round((num / 1000), 1))) else str(float(round((num / 1000), 1)))  + str('K')

        else:

            getcount = str(float(round((num / 1), 1)))[:-2] if '.0' in str(float(round((num / 1), 1))) else str(float(round((num / 1), 1)))

        return getcount

    def bytes(self, bit=0):
        # Return the given bytes as a human friendly KB, MB, GB, or TB string'
        B = float(bit)
        KB = float(1024)
        MB = float(KB ** 2)  # 1,048,576
        GB = float(KB ** 3)  # 1,073,741,824
        TB = float(KB ** 4)  # 1,099,511,627,776

        if B < KB:
            return '{0} {1}'.format(B, 'Bytes' if 0 == B > 1 else 'Byte')
        elif KB <= B < MB:
            return '{0:.2f} KB'.format(B / KB)
        elif MB <= B < GB:
            return '{0:.2f} MB'.format(B / MB)
        elif GB <= B < TB:
            return '{0:.2f} GB'.format(B / GB)
        elif TB <= B:
            return '{0:.2f} TB'.format(B / TB)

    def format(self, number=0):
        number = '{:0,.2f}'.format(float(number))
        return number



import re, os

class validation:

	def __getattr__(self, item):
		return item


	def __init__(self):
		return None
	
	def length(self, varstring, min, max=''):

		len_count = len(varstring)
		if min != "" and max != "":
			if len_count >= int(min) and  len_count <= int(max):
				return True
			else:
				return False
		if min != "":
			if len_count >= int(min):
				return True
			else:
				return False

	def count(self, varstring):

		return len(varstring)

	def email(self, varstring):

		regex = '^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$'
		if re.search(regex, varstring):
			return True
		else:
			return False 
			
	def url(self, varstring):

		regex = re.compile(
			r'^(?:http|ftp)s?://'  # http:// or https://
			r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
			r'localhost|'  # localhost...
			r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
			r'(?::\d+)?'  # optional port
			r'(?:/?|[/?]\S+)$', re.IGNORECASE)

		if regex.match(varstring):
			return True
		else:
			return False
		
	def ip(self, varstring):
     
		regex = '''^(25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[0-1]?[0-9][0-9]?)'''
		if re.match(regex, varstring):
			return True
		else:
			return False
		
  
	def single(self, varstring):
     
		regex = re.compile(r'^[a-zA-Z]+[a-zA-Z]+$', re.IGNORECASE)
		if regex.search(varstring):
			return True
		else:
			return False

	def phone(self, varstring):
     
		regex = re.compile(r'([+]?\d{1,4}[.\-\s]?)?(\d{3}[.-]?){2}\d{4}$')
		if regex.match(varstring):

			return True
		else:
			return False
 
	def number(self, varstring):
     
		regex = re.compile(r'(\d+)')
		if regex.match(varstring):
			return True
		else:
			return False

	def fullname(self, varstring):
     
		regex = re.compile(r'(^[a-zA-Z\- ]+)$', re.IGNORECASE)
		if regex.search(varstring):
			return True
		else:
			return False


	def trim(self, varstring):

		regex = re.sub('<[^<]+?>', '', varstring)

		return regex

	def extension(self, varstring, exe_list):
		string_text = os.path.splitext(varstring)[1][1:]
		if string_text in exe_list: 
			return True
		else:
			return False



from pytonik.Editor.Resolver import *

class _Fragment(object):

    def __init__(self, raw_text):

        self.raw = raw_text
        
        self.clean = self.clean_fragment()
        
        

    def clean_fragment(self):

        if self.raw[:2] in (VAR_TOKEN_START, BLOCK_TOKEN_START):

            return self.raw.strip()[2:-2].strip()

        return self.raw

    @property
    def type(self):
        raw_start = self.raw[:2]
        if raw_start == VAR_TOKEN_START:
            return VAR_FRAGMENT
        elif raw_start == BLOCK_TOKEN_START:

            block = str('end{}'.format(self.clean[3:9]))
            
            return CLOSE_BLOCK_FRAGMENT if self.clean[:9] == block else OPEN_BLOCK_FRAGMENT

        elif raw_start == COMMENT_TOKEN_START:
            if self.clean.find(TRANSLATOR_COMMENT_MARK):
                self.clean[2:-2].strip()
            return CLOSE_COMMENT_FRAGMENT
        else:

            return TEXT_FRAGMENT




# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 2/25/20.
from pytonik import Version, Config
import sys, os, re, pkg_resources
from pytonik.Log import Log


#Handles installation of pytonik project dependencies
class requirement:

    def __getattr__(self, item):
        return item

    def __init__(self):
        return None

    #The ``run``` method trigger Terminal command using `os` module and support both
    #windows, linux, mac os, ubuntu etc.  
    def run(self, pacakage, dir=""):
        dir_new = ' --install-option="--prefix={dir} "'.format(dir=dir) if dir != "" else ""
        cmd = "pip install {dir}{pacakage}".format(dir=dir_new, pacakage=pacakage)

        try:

            terminal = os.popen(cmd).read()
            print(terminal)
        except Exception as err:
            try:
                terminal =  os.system(cmd)
                print(terminal)
            except Exception as err:
                print(err)
                Log().critical(err)

    def rf(self, dir=""):

        if os.path.isdir(os.getcwd() + '/public'):
            host = os.getcwd()  # os.path.dirname(os.getcwd())

        else:
            host = os.path.dirname(os.getcwd())

        DS = str("/")

        requirepath = host + DS + "requirements.txt"
        if os.path.isfile(requirepath) == True:

            try:
                fb = open(requirepath, "r")
                ne_package = re.findall(r"\S+", fb.read())
                for l in ne_package:
                    self.check_package(l, dir)
                fb.close()
            except Exception as err:
                print(err)
                Log().error(err)
        else:
            print("Requirement file is not Found in this directory")



    def check_package(self, package, dir=""):

        installed_packages = pkg_resources.working_set
        installed_packages_list = sorted(["%s==%s" % (i.key, i.version) for i in installed_packages])
        package_n  = ""
        newpackage = package.split('==')[0]
        lis = []
        for packall in installed_packages_list:
            lis.append(str(str(packall).split('==')[0]).lower())


        if str(newpackage).lower() not in lis :
            self.run(package, dir)


###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###

import sys
import os
from pytonik.Version import *
from pytonik.Log import Log
from pytonik.Config import Config
from pytonik.Core.env import env
from pytonik.Session import Session
from pytonik.util.Variable import Variable
from .Core import Helpers

h = Helpers

if os.path.isdir(os.getcwd() + '/public'):
    host = str(os.getcwd()).replace('\\', '/')  # os.path.dirname(os.getcwd())

else:
    host = str(os.path.dirname(os.getcwd())).replace('\\', '/')

DS="/"
class Controllers(env, Config):
    def __getattr__(self, item):
        return item

    def __init__(self):

        self.uri = self.url()

        self.add(self._e())

        self.controllers = self.get('default_controllers')
        self.actions = self.get('default_actions')
        self.default_controllers = self.get('default_controllers')
        self.default_actions = self.get('default_actions')

        self.languages = self.get('default_languages')
        self.all_languages = self.get('languages', '')
        self.routes = self.get('default_routes', "")
        self.methodprefix = ""
        self.parameter = {}

        path_parts = self.uri

        # Get parameter from Query String
        pathparts_paramarray = os.environ.get("QUERY_STRING", '')

        pathparts_paramarrayOut = dict()
        if pathparts_paramarray != '':
            pairs = pathparts_paramarray.split('&')

            pathparts_paramarray = pairs

            for i in pairs:

                name, value = i.split('=', 2)

                pathparts_paramarray = {name: value}

                if name in pathparts_paramarray:

                    pathparts_paramarray[name] = value

                else:
                    pathparts_paramarray[name] = [
                        pathparts_paramarray[name], value]

                pathparts_paramarrayOut.setdefault(name, value)

        else:
            pathparts_paramarrayOut = ""

        if len(path_parts):

            if PYVERSION_MA < 3:
                path_parts = filter(None, path_parts)
            else:
                path_parts = list(filter(None, path_parts))

            routes = self.get('route', '')
            try:
                if list(set(path_parts).intersection(routes.keys())):

                    for s in path_parts:

                        if s in routes:
                            self.routes = s

                            if self.routes in routes:
                                self.methodprefix = routes[self.routes]
                            else:
                                self.methodprefix = ""

                                # path_parts.append(path_parts.pop(-1))

            except Exception as err:
                Log("").error(err)

            try:
                if list(set(path_parts).intersection(self.all_languages.keys())):

                    for s in path_parts:

                        if s in self.all_languages:
                            self.languages = s
                            path_parts.pop(0)
                            #path_parts.append()

            except Exception as err:
                Log("").error(err)

            controllers = self.get('default_controllers', '')
            try:
                if controllers:

                    i = 0
                    path_parts = list(filter(None, path_parts))

                    for s in path_parts:

                        if s is not self.languages:
                            i += 1

                            if i == 1:
                                self.controllers = s
                                path_parts.append(path_parts.pop(-1))
                        ++i
            except Exception as err:
                Log("").error(err)

            action = self.get('default_actions', '')
            if action:
                i = 0
                for s in path_parts:
                    if s is not self.controllers and s is not self.languages:
                        i += 1
                        if i == 1:
                            self.actions = s
                            path_parts.append(path_parts.pop(-1))
                        ++i

            # Get Path from URI / convert it to parameter
            list_params = []

            try:

                if routes.get(self.controllers) != "":
                    if PYVERSION_MA <= 2:
                        lroutes = routes.iteritems()
                    else:
                        lroutes = routes.items()

                    for k, getRouter in lroutes:

                        if self.controllers == k:

                            paraUri = getRouter.split('@')
                        else:
                            paraUri = []
                        if len(paraUri) > 0:
                            if ':' not in paraUri[1]:
                                getMapPara = []
                            else:
                                getMapPara = paraUri[1].split(':')

                            if self.controllers in routes:

                                if len(getMapPara[1:]) > 0:

                                    new_para = path_parts[1:]

                                    if len(new_para) > 0:
                                        param_m = []

                                        for i, para in enumerate(getMapPara[1:]):

                                            param_n = para

                                            if (len(new_para) - i) > 0:

                                                v_para = self._removeparseurl(new_para[i])
                                                self._parseurl(new_para[i])
                                            else:
                                                v_para = ""

                                            list_params.append(param_n)
                                            list_params.append(v_para)

                                    self.parameter.update(Helpers.covert_list_dict(
                                        list_params))
                                break

                    else:
                        for s in path_parts:

                            if s is not self.controllers and s is not self.actions and s is not self.languages:
                                list_params.append(self._removeparseurl(s))

                                path_parts.append(path_parts.pop(-1))

                        self.parameter.update(Helpers.covert_list_dict(list_params))

                else:

                    self.parameter.update(pathparts_paramarrayOut)
                    path_parts.append(path_parts.pop(-1))

            except Exception as err:
                Log("").error(err)

        return None

    def url(self):
        osv = Variable()
        url = osv.out('REQUEST_URI', "")
        http_s = osv.out("HTTP_HOST")
        uri = ""
        if osv.out("SERVER_SOFTWARE", "") == AUTHOR:

            uri = url.split('/')[2:]

        else:

            if http_s == "127.0.0.1" or http_s == "localhost":
                uri = url.split('/')[2:]
            else:
                uri = url.split('/')[1:]
        return uri

    def _getUri(self):
        return self.uri

    def _getControllers(self):
        return self.controllers

    def _getActions(self):
        return self.actions

    def _getLanguages(self):
        return self.languages

    def _getParams(self):
        return self.get_routes_param(params=self.parameter)

    def _getMethodPrefix(self):

        return self.methodprefix

    def _getRoutes(self):
        return self.routes

    def get_routes_param(self, params):
            list_params = []
            if os.path.isfile(host + "/" + "routes.py") == True:
                sys.path.append(host)
                import routes as route
                param_v = {}

                lsr = filter(lambda x : x !="", route.route.getParams())
                if len(list(lsr)) > 0:

                    for i, route_c in enumerate(route.route.getRouter()):
                        if self.controllers == route_c:
                            param_v = self._getR_params(route.route.getParams()[i])
                            break

                    if len(param_v) > 0:
                        return param_v
                    else:
                        return params
                else:
                    return params

            else:
                return params

    def _getR_params(self, params):
        list_params = []
        parameter = {}


        new_uri = self._getUri()[1:]

        for i, para in enumerate(params):

            try:
                v_para = self._removeparseurl(new_uri[i])
            except Exception as err:
                v_para = ""
            list_params.append(para)
            list_params.append(v_para)


        parameter = Helpers.covert_list_dict(list_params)
        return parameter

    def _removeparseurl(self, _url):

        if '/?' in _url:
            url_s = str(_url).split('/?')
            url_ = _url.replace('/?', '')
        elif '?' in _url:
            url_s = str(_url).split('?')
            url_ = _url.replace('?', '')
        else:
            url_ = _url.replace('', '')
            url_s =  []

        if len(url_s) > 1:
            pairs = str(url_s[1]).split('&')
            url_ = _url.replace('&', '').replace('/?', '').replace('?', '')
            for i in pairs:
                url_ = url_.replace(i, '')
        else:
            url_ = _url
        return url_

    def _parseurl(self, _url):

        if '/?' in _url:
            url_s = str(_url).split('/?')
        elif '?' in _url:
            url_s = str(_url).split('?')
        else:
            url_s = []

        if len(url_s) > 1:
            pairs = str(url_s[1]).split('&')

            for i in pairs:
                try:
                    name, value = i.split('=', 2)
                except Exception as err:
                    name = i.split('=')
                    value = ""
                self.parameter.update({name:value})

        return self.parameter




# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 05/11/2019.
import os
from pytonik.util.Variable import Variable
from pytonik.Router import Router
from pytonik import Version


class url(Variable):
    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):
        return None

    def __init__(self, *args, **kwargs):

        if len(args) > 0 or len(kwargs) > 0:

            if all(args) is not False:
                self.ul = self.url(*args, **kwargs)
            else:
                self.ul = self.url(**kwargs)
        else:
            self.ul = self.url(*args)
        return None

    def __str__(self):

        return self.ul

    def url(self, path="", lang=False):

        DS, p = "", ""
        self.Router = Router()
        seturl = self.out("HTTP_HOST")+str(":")+str(self.out("SERVER_PORT", '')) if self.out("HTTP_HOST") == "localhost" or self.out("HTTP_HOST") == "127.0.0.1" else self.out("HTTP_HOST")

        if self.out("HTTPS", "") == 'on':
            url = str("https://") + seturl.replace(":80", "") + "/" + self.Router.alllanguages.get(self.Router.getLanguages(), self.Router.getLanguages()) if lang == True else str(
                    "https://") + seturl.replace(":80", "") 
        else:

            url = str("http://") + seturl.replace(":80", "")+ "/" + self.Router.alllanguages.get(self.Router.getLanguages(), self.Router.getLanguages()) if lang == True else str("http://") + seturl.replace(":80", "")

        if path == "":
            DS = ""
        else:

            if path[:1] == "/":
                p = path[1:]
                DS = "/"
            else:
                p = path
                DS = "/"

        return url + DS + p



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import os
import cgi
from pytonik.Controllers import Controllers
from pytonik.util.Variable import Variable
from pytonik.Version import *
from pytonik.Log import Log


class Request(Variable):

    def __getattr__(self, item):
        return item

    def __init__(self, prform=None):
        self.Controllers = Controllers()
        self.para_vv ={} 
        if self.out("SERVER_SOFTWARE") == AUTHOR:
             self.attr = prform
        else:
            self.attr = cgi.FieldStorage()
        self.method = self.out('REQUEST_METHOD', '')

    def get(self, key=0, error=0):
        
        try:
            if 'GET' in self.out('REQUEST_METHOD'):

                if key != 0:
                    if (key in self.attr):
                        return self.attr.getvalue(key)
                    elif error == 1:
                        return self.attr
                    else:
                        return self.params(key)
                else:
                    return ""
            else:
                
                Log('').warning("advise use POST instead of GET")
                return False
        except Exception as err:
            Log('').warning(err)
            return err

    def post(self, key=0, error=0):
        try:
            if 'POST' in self.out('REQUEST_METHOD'):
                if key != 0:
                    if (key in self.attr):
                        return self.attr.getvalue(key)
                    elif error == 1:
                        return self.attr
                    else:
                        return ""
                else:
                    return ""
            else:
                Log('').warning("advise use GET instead of POST")
                return False
        except Exception as err:
            Log('').warning(err)
            return err

    def file(self, key=0, error=0):
        try:
            if key != 0:
                if (key in self.attr):
                    self.attr.getvalue(key)
                    return self.attr[key]
                elif error == 1:
                    return self.attr
                else:
                    return ""
            else:
                return ""

        except Exception as err:
            Log('').warning(err)
            return err

    def all(self):
        if None is not self.attr.keys():
            return self.attr.keys()

    def params(self, key=""):
        self.para_vv = self.Controllers._getParams()
        if key == "" or key == None:
            result = self.para_vv  
        else:
            result =  self.para_vv.get(key, '') 
        return result       
       
        



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 08/11/2019.
#############################################
#############################################
# SQLITE Support Database Connection
# Using Raw query from sqlite3 module
# Each method represent extensions from SQLITE 3


from pytonik import Log
import os, sys
log_msg = Log.Log()
host = os.path.dirname(os.getcwd())
D = "/"
try:
    import sqlite3

except Exception as err:
    log_msg.critical(err)



class SQLite:

    def __init__(self, setting):
        self.path = setting.get('path', '')
        self.name = setting.get('name', '')
        self.bdfile = str(host) + D + str(self.path)+ D +str(self.name)
        self.prefix = setting.get('prefix', '')
        self.Exception = ""
        self.success = ""
        self.conn = None
        self.con = None
        self.result = None
        self.connectDB()
        

    def connectDB(self):

        try:
            self.conn = sqlite3.connect(self.bdfile)
         
        except Exception as err:
            log_msg.error(err)
            self.Exception = err


    def query(self, sql="", value = ""):
        try:
            self.con = self.conn.cursor()
            if sql !="" and value != "":
                self.con.execute(str(sql), value)
            else:
                self.con.execute(str(sql))
        except Exception as err:
            log_msg.error(err)
            self.Exception = err        
        return self

    def querymultiple(self, sql="", value = ""):
        try:
            self.con = self.conn.cursor()
            if sql !="" and value != "":
                self.con.executemany(str(sql), value)
            else:
                self.con.executemany(str(sql))
        except Exception as err:
            log_msg.error(err)
            self.Exception = err   
        return self


 
    def lastId(self):
        return self.con.lastrowid

    def fetch(self):
        result = self.con.fetchall()
        row = []
        for r in result:
            rowf = {}
            for idx, col in enumerate(self.con.description):
                rowf[col[0]] = r[idx]
            row.append(rowf)
        return row if len(row) > 0 else ""

    def queryone(self, sql="", value = ""):
        self.con = self.conn.cursor()
        if sql !="" and value != "":
            
            self.con.execute(str(sql), value)
        else:
            self.con.execute(str(sql))

        return self.con

    def all(self):
        self.result = self.fetch()
        return self.result

    def one(self):
        result = self.con.fetchone()
        row = {}
        for idx, col in enumerate(self.con.description):
            
            row[col[0]] = result[idx]
        return row

    def count(self):
        return self.con.rowcount

    def countall(self):
        self.all()
        return self.con.rowcount if self.con.rowcount > 0 else 0
     
    def save(self):
        try:
            self.conn.commit()
            return True
        except Exception as err:
            self.Exception = err
            log_msg.error(err)
            return self

    def close(self):
        return self.con.close()


    def create(self, TABLES = ''):
        self.con = self.conn.cursor()
        if TABLES:
            for table_name in TABLES:
                table_description = TABLES[table_name]
                try:
                    self.con.execute(table_description)
                    self.Exception = "Database table '{}' created successfully.".format(table_name)
                except Exception as err:
                
                    log_msg.info("Database table '{}' already exists.".format(table_name))
                    self.Exception =  "Database table '{}' already exists.".format(table_name)
            
            return self
        
        else:
            self.Exception =  "Empty Table"
            return self




###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


from pytonik.Request import Request
from pytonik.Session import Session
from pytonik.Editor import Template
from pytonik.Config import Config
from pytonik.Log import Log
from pytonik import Lang
from pytonik.Version import *
from pytonik.Core.env import env
from pytonik.util.Variable import Variable
from pytonik.Functions import url
from pytonik.Controllers import Controllers
import os
import sys
import cgi
import cgitb
import importlib
import glob
import  json
import inspect

cgitb.enable()

global host, u

if os.path.isdir(os.getcwd() + '/public'):
    host = os.getcwd()  # os.path.dirname(os.getcwd())

else:
    host = os.path.dirname(os.getcwd())

DS = str("/")
u = url

controllerpath = host + DS + 'controller'
error_page_class = controllerpath + DS + "ErrorController" + ".py"
header_response_page = {
    '301': 'error/page404',  # Bad Request
    '302': 'error/page404',  # Bad Request
    '307': 'error/page404',  # Bad Request
    '400': 'error/page400',  # Bad Request
    '404': 'error/page404',  # Not Found
    '405': 'error/page405',  # Method Not Allowed
    '500': 'error/',  # Internal Server Error

}

data_c = {}

class App(env, Config, Variable):

    def __getattr__(self, item):
        return item

    def __init__(self, routers="", routes="", getpath="", getrouters=""):

        self.routers = routers
        self.routes = routes
        self.getpath = getpath
        self.codereponse = 200
        self.getrouters = getrouters
        self.getDB = ""
        self.Request = Request
        self.Session = Session
        self.methodprefix = ""
        self.actions = ""
        self.controllers = ""
        self.controller = ""
        self.dbDriver = None
        self.Config = None
        self.url = ""
        self.params = ""
        self.key = ""
        self.vpathf = ""
        self.languages = ""
        self.Driver = ""
        self.formData = None
        self.datag = {}

    def getRouters(self):

        return self.routers

    def getPath(self):
        return self.getpath

    def runs(self, formData=None):

        self.formData = formData
        return self.initial()

    def initial(self):
        self.default("Framework", AUTHOR)
        self.default("X-Version", VERSION_TEXT)
        self.getrouters = self.envrin('route')
        self.controllers = Controllers()
        self.methodprefix = self.controllers._getMethodPrefix()
        self.actions = self.controllers._getActions()
        self.uri = self.controllers._getUri()
        self.controller = self.controllers._getControllers()
        self.routers = self.controllers._getRoutes()
        self.languages = self.controllers._getLanguages()
        self.params = self.controllers._getParams()
        langs = Lang.Lang(self.languages)
        langs.loadLang()

        controlUri = []


        if PYVERSION_MA >= 3:
            lrounters = self.getrouters.items()
        else:
            lrounters = self.getrouters.iteritems()

        for k, getRouter in lrounters:

            if self.controller == k:
                controlUri = getRouter.split('@')

        self.routerend()

        if len(controlUri) != 0:

            if 'controller' in controlUri[0].lower():

                controllersClass = str(controlUri[0].lower()).replace(
                    'controller', '').capitalize() + 'Controller'
            else:
                controllersClass = str(
                    controlUri[0]).capitalize() + 'Controller'

            if ':' not in controlUri[1]:
                controllersMethods = str(controlUri[1])

            else:
                getMapPara = controlUri[1].split(':')

                controllersMethods = str(getMapPara[0])

        else:

            if str(self.controller[0]) == '?':
                if self.controllers.default_controllers != "":
                    controllersClass = str(self.controllers.default_controllers).capitalize() + 'Controller'

                else:
                    controllersClass = "IndexController"
            else:

                controllersClass = str(self.controller[0]).capitalize(
                ) + str(self.controller[1:]) + 'Controller'

            controllersMethods = str(self.actions)

        if controllersMethods != "":
            self.actions = controllersMethods

        controllers = controllerpath + DS + controllersClass + ".py"

        if os.path.isfile(controllers) == True:

            if __name__ == '__main__':
                spac = ""
            if os.path.isfile(controllers) == True:

                if sys.version_info.major <= 2:

                    return self.strClass(controllerpath, controllersClass)
                else:

                    return self.strClass3(controllerpath, controllersClass)

        else:
            Log(controllerpath).error(
                'Controller does not exist ' + str(controllersClass))
            return self.errorP('405')

    def errorP(self, code, replace=""):
        getErrorP = self.envrin('error')

        if os.path.isfile(host + DS + "routes.py") == True:
            sys.path.append(host)
            import routes as route

            if len(route.route.getError()) > 0:

                for i, redirect in enumerate(route.route.getError()):

                    if len(route.route.getError()) > 0:
                        toeroor = route.route.getError()[i]
                    if len(route.route.getErrorControl()) > 0:
                        fromroute = route.route.getErrorControl()[i]

                    if len(route.route.getCode()) > 0:
                        code = route.route.getCode()[i]

                    if len(route.route.getLink()) > 0:
                        link = route.route.getLink()[i]

                    if fromroute == self.controller:
                        return self.redirect(location=toeroor, link=bool(link), code=code)

        pageCode = "page{code}".format(code=code)
        if os.path.isdir(os.getcwd() + '/public'):
            if self.out("SERVER_SOFTWARE") == AUTHOR:
                re_url = ""
                errorP = {}
                if getErrorP != '':
                    errorP = getErrorP

                if errorP.get(code, '') != '':

                    controllerP = ""
                    if '/' in errorP.get(code, ''):
                        splitP = errorP.get(code, '').split('/')
                        controllerP = controllerpath + DS + \
                            str(splitP[0]).capitalize() + 'Controller' + ".py"
                    else:
                        controllerP = controllerpath + DS + str(
                            errorP.get(code, '')).capitalize() + 'Controller' + ".py"

                    if code == "500":

                        if os.path.isfile(controllerP) == True:
                            re_url = u.url().url('/' + str(errorP.get(code, '')))
                        else:
                            re_url = u.url().url("/error")
                    else:
                        if os.path.isfile(controllerP) == True:
                            re_url = u.url().url('/' + str(errorP.get(code, '')))
                        else:

                            if os.path.isfile(self.error_page_html(code)) == True:
                                re_url = u.url().url(
                                    "/error/{code}".format(code=pageCode))

                else:
                    if code == "500":
                        re_url = u.url().url("/error")
                    else:

                        if os.path.isfile(self.error_page_html(code)) == True:
                            re_url = u.url().url(
                                "/error/{code}".format(code=pageCode))

                self.put(status=code)

                return code, re_url

        if getErrorP != '':

            errorP = getErrorP

            if errorP.get(code, '') != '':
                self.put(status=code)
                if '/' in errorP.get(code, ''):
                    splitP = errorP.get(code, '').split('/')
                    controllerP = controllerpath + DS + \
                        str(splitP[0]).capitalize() + 'Controller' + ".py"
                else:
                    controllerP = controllerpath + DS + str(
                        errorP.get(code, '')).capitalize() + 'Controller' + ".py"

                if os.path.isfile(controllerP) == True:
                    return self.redirect(u.url().url('/' + str(errorP.get(code, ''))))
                else:
                    if os.path.isfile(self.error_page_html(code)) == True:
                        return self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

            else:

                if os.path.isfile(self.error_page_html(code)) == True:
                    return self.redirect(u.url().url("/error/{code}".format(code=pageCode)))
            self.default("REDIRECT_STATUS", 301)
            self.default("REDIRECT_REDIRECT_STATUS", 301)
        else:
            if os.path.isfile(error_page_class) == True:
                self.default("REDIRECT_STATUS", 301)
                self.default("REDIRECT_REDIRECT_STATUS", 301)
                return self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

    def envrin(self, key):
        env = self._e()
        self.add(env)
        return self.get(key, '')

    def DB(self):

        self.dbDriver = self.envrin('dbConnect')
        if self.dbDriver.get("driver", '') == "MYSQL":
            from pytonik.Driver.DB.MYSQL.MYSQL import MYSQL
            self.getDB = MYSQL(self.dbDriver)
            self.Driver = "MYSQL"
            return self.getDB

        if self.dbDriver.get("driver", '') == "SQLite":
            from pytonik.Driver.DB.SQLite.SQLite import SQLite
            self.getDB = SQLite(self.dbDriver)
            self.Driver = "SQLite"
            return self.getDB

        if self.dbDriver.get("driver", '') == "Oracle":
            from pytonik.Driver.DB.Oracle.Oracle import Oracle
            self.getDB = Oracle(self.dbDriver)
            self.Driver = "Oracle"
            return self.getDB

        if self.dbDriver.get("driver", '') == "pyPgSQL":
            from pytonik.Driver.DB.pyPgSQL.pyPgSQL import pyPgSQL
            self.getDB = pyPgSQL(self.dbDriver)
            self.Driver = "pyPgSQL"
            return self.getDB

    def strClass(self, p=None, c=None):

        try:
            sys.path.append(p)
            ms = str(self.actions)
            md = importlib.import_module(c)
            self.default("REDIRECT_REDIRECT_STATUS", 200)
            self.default("REDIRECT_STATUS", 200)
            return self.strMethod(p, md, ms)

        except Exception as err:
            Log(p + DS + c + '.py').critical(err)

            return self.errorP('400')

    def strMethod(self, p, c=None, mv=None):
        gmodule = []
        Request = self.Request(prform=self.formData)
        Session = self.Session()
        
        try:
            m = mv.split("?")[0]
        except Exception as err:
            m = mv
        try:
            return getattr(c, m)(Request, Session)
        except Exception as err:
            try:
                return getattr(c, m)(Session, Request)
            except Exception as err:
                try:
                    return getattr(c, m)(Request)
                except Exception as err:
                    try:
                        return getattr(c, m)(Session)
                    except Exception as err:
                        try:
                            return getattr(c, m)()
                        except Exception as err:
                            Log(p + DS + c + '.py').critical(err)
                            return self.errorP('400')

    def strClass3(self, p=None, c=None):

        try:
            sys.path.append(p)
            ms = str(self.actions)
            importlib._RELOADING
            md = importlib.import_module(c, ms)
            self.default("REDIRECT_REDIRECT_STATUS", 200)
            self.default("REDIRECT_STATUS", 200)
            return self.strMethod(p, md, ms)

        except Exception as err:


            Log(p + DS + c + '.py').critical(err)
            return self.errorP('400')

    def redirect(self, location='/', link=False, code="307"):

        if self.out("SERVER_SOFTWARE") == AUTHOR:

            if link == True:
                location_d = u.url().url(location)
            else:
                location_d = location
            self.put(status=code)
            self.put(redirect_url=location_d)
            return code, location_d
        else:

            if link == True:
                location_d = u.url().url(location)
            else:
                location_d = location

            self.default("REDIRECT_STATUS", code)
            self.default("REDIRECT_REDIRECT_STATUS", code)
            print("Location: {location}".format(location=location_d))
            print()

    def referer(self, location='/', link=False, code="307"):
        if self.out("SERVER_SOFTWARE") == AUTHOR:
            if link == True:
                location_d = u.url().url(location)
            else:
                location_d = location
            self.put(status=code)
            self.put(referral=self.out('HTTP_REFERER', location_d))
            return code, self.out('HTTP_REFERER', location_d)
        else:
            if link == True:
                location_d = u.url().url(location)
            else:
                location_d = location
            self.default("REDIRECT_STATUS", code)
            self.default("REDIRECT_REDIRECT_STATUS", code)
            print("Location: {location}".format(
                location=self.out('HTTP_REFERER', location_d)))
            print()

    @staticmethod
    def header(p=0, type="text/html"):

        try:
            import imp
            imp.reload()
        except Exception:
            import importlib
            importlib.reload(sys)

        if os.path.isdir(os.getcwd() + '/public') == False:
            print("Content-type: {type}\r\n".format(type=type))  # \r\n\r\n
            if p > 0:
                for x in range(p):
                    print("")
        else:
            return

    def Jdumps(self, strings):

        try:
            return json.dumps(strings)
        except Exception as err:
            return strings

    def Jloads(self, strings):

        try:
            return json.loads(strings)
        except Exception as err:
            return strings


    def XHreponse(self, dataString, ctype=""):
        if self.out("SERVER_SOFTWARE") == AUTHOR:
            return dataString
        else:

            for mime in MIME_TYPES:
                if str(mime["ext"]).replace(".", "") == ctype or str(mime["ext"]).replace(".", "") == ctype:
                    ctype = mime["type"]
            self.header(type=ctype)
            print(dataString)

    def views(self, pathf="", datag={}, datal={}):
        data_c.update(datag)

        if pathf == "":
            pathf = self.getDefaultViewPath()

        pathfhtml = host + DS + 'views' + DS + pathf + ".html"
        html = ""
        if os.path.isfile(pathfhtml) == False:
            Log(host + DS + 'views').critical('Cannot find file {}'.format(pathf + ".html"))

            return self.errorP('404')

        else:
            output = str('<!-- Pytonik -->\n') + self.read_html(host + DS + 'views' + DS, pathf, data_c) + str(
                '\n<!-- Pytonik {} -->'.format(VERSION_TEXT))
            if os.path.isdir(os.getcwd() + '/public'):
                # print(os.environ)
                return output

            else:
                self.header()
                print(output)



    def read_html(self, template_dir, engine, context={}):
        data_c.update(context)
        html_file_path = os.path.join(template_dir, "%s.html" % engine)
        try:
            with open(html_file_path, encoding='utf-8') as html_file:
                html = html_file.read()

            return Template.Template(html).render(**data_c)

        except Exception as err:

            Log(template_dir + DS + engine + str('.html')).error(err)
            return ""

    def getDefaultViewPath(self):

        router = self.Router.getRoutes()  # Routers.getRoutes()

        if router == "":
            rout = ""
        controllerDirectory = self.Router.getControllers()

        templateName = str(self.Router.getMethodPrefix()) + \
            str(self.Router.getAction()) + '.html'

        return templateName

    def error_page_html(self, code):
        return host + DS + 'views' + DS + str(code) + ".html"

    def loadmodule(self):

        path = [os.path.dirname(__file__) + str("/") +
                str("Functions"), str(host) + str("/") + "model"]
        listpath = [path[0], path[1]]

        i = 0
        lclass = {}

        for pl in listpath:

            current_dir = os.path.join(pl)

            current_module_name = os.path.splitext(
                os.path.basename(current_dir))[0]

            for file in glob.glob(os.path.join(current_dir + "/*.py")):
                name = os.path.splitext(os.path.basename(file))[0]

                i = i + 1
                # Ignore __ files
                ++i
                if name.startswith("__init__"):
                    continue
                if name != "__init__":
                    lclass0 = {name: name}
                    lclass.update(lclass0)

        if PYVERSION_MA <= 2:
            item = lclass.iteritems()
        else:
            item = lclass.items()
        result = {}
        for key, value in item:
            if value not in result.values():
                result[key] = value

        return result

    def routerend(self):
        if os.path.isfile(host + DS + "routes.py") == True:
            sys.path.append(host)
            import routes as route
            toroute, fromroute = "", ""

            if len(route.route.getRouter()) > 0:

                for i, redirect in enumerate(route.route.getCode()):
                    if len(route.route.getRediret()) > 0:
                        toroute = route.route.getRediret()[i]
                    if len(route.route.getDespatch()) > 0:
                        fromroute = route.route.getDespatch()[i]
                    if fromroute == self.controller:

                        self.redirect(location=toroute,
                                      link=True, code=redirect)

                for i, route_c in enumerate(route.route.getRouter()):

                    uri = self.uri
                    while("" in uri):
                        try:
                            uri.clear("")
                        except Exception as err:
                            uri.remove("")


                    if self.languages in uri:
                        uri.pop(0)

                    sltp  = str(route_c).split("/")
                    luri = ""
                    if len(uri) > 0:

                        luri = "/".join(uri[0:len(sltp)])

                    else:
                        luri = str(uri)

                    if luri == route_c:
                        routes_l = route_c


                        if len(route.route.getController()) > 0:
                            self.controller = route.route.getController()[i]

                        if len(route.route.getAction()) > 0:
                            self.actions = route.route.getAction()[i]

                        if len(route.route.getRouter()) > 0:
                            self.routers = route.route.getRouter()[i]
                        if len(route.route.getMethod()) > 0:
                            self.method = route.route.getMethod()[i]

                            if self.method != "":
                                if self.method != self.out("REQUEST_METHOD", ""):
                                    Log().error(str(str(self.controller).capitalize()) + "Controller/" + str(
                                        self.actions) + " Requires " + self.method)

                                    return self.errorP('400')




###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###

__path__ = __import__('pkgutil').extend_path(__path__, __name__)



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 13/01/2020.


import sys

from pytonik.cmd import start
from pytonik.cmd import server
from pytonik.cmd import doc
from pytonik.cmd import install
from pytonik.cmd import help

if __name__ == '__main__':
    sys.exit(start.main(sys.argv[1:]))

if __name__ == '__main__':
    sys.exit(server.main(sys.argv[1:]))

if __name__ == '__main__':
    sys.exit(doc.main(sys.argv[1:]))

if __name__ == '__main__':
    sys.exit(install.main(sys.argv[1:]))

if __name__ == '__main__':
    sys.exit(help.main(sys.argv[1:]))




###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2029.
###
from pytonik.Session import Session
from pytonik.util.Variable import Variable
from pytonik.Controllers import Controllers
FSession = Session()
FVariable = Variable()
FControllers = Controllers()

class Flash:

    def __getattr__(self, item):
        return item

    def __init__(self):
        return 
        
    @staticmethod
    def message(message, showin="", key='flash'):
        option = {'msg': message, 'controller' : showin if showin != "" else "/".join(str(FVariable.out('HTTP_REFERER')).split('/')[3:])}  
        if FSession.has(key) == True:
            FSession.destroy(key)
            return FSession.set(key, option)
        else:
            return FSession.set(key, option)

    @staticmethod
    def display(key='flash'):
        if FSession.has(key) == True:
            option = dict(FSession.get(key))
            if option.get('controller') == "/".join(FControllers._getUri()):
                return option.get('msg')
            else:
                return ""
        else:
            return ""
    
    


# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 1/19/20.



import cgitb
import base64
import traceback
import socket
import argparse
import locale
import os
import re
import sys
import time
import warnings
from pytonik.cmd import lang
from pytonik.cmd import start
from pytonik.cmd import server
from pytonik.cmd import install
from pytonik.cmd import doc
from pytonik import Version
from pytonik import serv 
from pytonik.Functions.url import url
from typing import Any, Callable, Dict, List, Pattern, Union
from pytonik.cmd.console import (  # type: ignore
    colorize, bold, red, green, turquoise, nocolor, color_terminal
)


import webbrowser

cgitb.enable()


try:
    import readline

    if readline.__doc__ and 'libedit' in readline.__doc__:
        readline.parse_and_bind("bind ^I rl_complete")
        USE_LIBEDIT = True
    else:
        readline.parse_and_bind("tab: complete")
        USE_LIBEDIT = False
except ImportError:
    USE_LIBEDIT = False


class ValidationError(Exception):
    """Raised for validation errors."""


def is_path(x: str) -> str:
    x = os.path.expanduser(x)
    if not os.path.isdir(x):
        raise ValidationError(__("Please enter a valid path name."))
    return x


def boolean(x: str) -> bool:
    if x.upper() not in ('Y', 'YES', 'N', 'NO'):
        raise ValidationError(__("Please enter either 'y' or 'n'."))
    return x.upper() in ('Y', 'YES')


def allow_empty(x: str) -> str:
    return x


def nonempty(x: str) -> str:
    if not x:
        raise ValidationError(__("Please enter some text."))
    return x


def __(mes_id):
    try:
        userlang = locale.getlocale()[0]

        l_ = mes_id
        for l in lang.lang:
            if l == str(userlang):
                getla = lang.lang.get(l, '')

                if getla != "":
                    l_ = getla.get(mes_id, '')
                    if l_ != "":
                        l_ = getla.get(mes_id, '')
                    else:
                        l_ = mes_id
                else:
                    l_ = mes_id
    except Exception as arr:
        l_ = mes_id
    return l_


def doTraceBack():
    exc_type, exc_value, exc_traceback = sys.exc_info()
    traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
    traceback.print_exception(exc_type, exc_value, exc_traceback,
                              limit=2, file=sys.stdout)
    traceback.print_exc()


if sys.platform == 'win32':
    # On Windows, show questions as bold because of color scheme of PowerShell
    # (refs: #5294).
    COLOR_QUESTION = 'bold'
else:
    COLOR_QUESTION = 'purple'

PROMPT_PREFIX = '> '


def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] = nonempty) -> Union[str, bool]:  # NOQA

    while True:

        if default is not None:
            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
        else:
            prompt = PROMPT_PREFIX + text + ': '
        if USE_LIBEDIT:
            # Note: libedit has a problem for combination of ``input()`` and escape
            # sequence (see #5335).  To avoid the problem, all prompts are not colored
            # on libedit.
            pass
        else:
            prompt = colorize(COLOR_QUESTION, prompt, input_mode=True)
        x = term_input(prompt).strip()
        if default and not x:
            x = default
        try:
            x = validator(x)
        except ValidationError as err:
            print(red('* ' + str(err)))
            continue
        break
    return x


def term_input(prompt: str) -> str:
    if sys.platform == 'win32':

        print(prompt, end='')
        return input('')
    else:
        return input(prompt)


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        usage='%(prog)s [OPTIONS] <PROJECT_DIR>',
        epilog=__(
            "For more information, visit < https://pytonik.readthedocs.io >."),
        description=__("""Pytonik is a python framework built to enhance web development
        fast and easy, also help web developers to build more apps with less codes.
        it uses expressive architectural pattern, structured on model view controller MVC
        and bundles of component to reuse while deploying the framework.
        
        
        """))

    parser.add_argument('-q', '--quit', action='store_true', dest='quit',
                        default=None,
                        help=__('quit mode'))

    parser.add_argument('--version', '-v', action='version', dest='show_version',
                        version='Pytonik %s' % Version.VERSION_TEXT)

    parser.add_argument('start', metavar='PROJECT_DIR', default='all', nargs='?',
                        help=__('Terminal Command'))
    
    return parser


def ask(d: Dict) -> None:
    
    if len(d) > 0:
        if d.get('start', '') == 'start':
            return start.main(sys.argv[1:])

        if d.get('start', '') == 'server':
            return server.main(sys.argv[1:])

        if d.get('start', '') == 'install' :
            return install.main(sys.argv[1:])
        
        if d.get('start', '') == 'version' :
            print('Pytonik %s' % Version.VERSION_TEXT)
        
        if d.get('start', '') == 'doc' or d.get('start', '') =="docs":
            return doc.main(sys.argv[1:])
        
        if d.get('start', '') == 'quit':
            return sys.exit(-1)
        

        if d.get('start', '') == 'all':
            return help_notes()

        if d.get('quit', '') is True:
            return sys.exit(-1)
            
    else:
        return help_notes()
    

def help_notes():
    cnt_text = """
    Pytonik is a python framework built to enhance web development
    fast and easy, also help web developers to build more apps with less codes.
    it uses expressive architectural pattern, structured on model view controller MVC
    and bundles of component to reuse while deploying the framework.

    usage: 
    Options and arguments (and corresponding environment variables):

    pytonik [option]
    ================
    start     			: Create new pytonik project to an assign directory
    server     			: Run pytonik Server from an assign project directory
    install 			: Install pytonik requirement from an assign directory
    doc     			: Read Pytonik Documentation (doc) or (docs)
    version     		: Pytonik Version (-v) or (--version)

    direct command [pytonik-option]
    ===============================
    pytonik-start     	: Create a new pytonik project to an assign directory
    pytonik-server     	: Run pytonik Server from an assign project directory
    pytonik-install     : Install pytonik requirement from an assign directory
    pytonik-docs     	: Read Pytonik Documentation
    """

    print(cnt_text)

def main(argv: List[str] = sys.argv[1:]) -> int:
    # parse options
    parser = get_parser()
    try:
        args = parser.parse_args(argv)

    except SystemExit as err:
        return err.code
    d = vars(args)
    ask(d)

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import os
import datetime
import sys
import ast
from pytonik.Version import *
from pytonik.Log import Log
from pytonik.util.Variable import Variable


try:
    from http import cookies as cook
except Exception as err:
    import Cookie as cook


class Session(Variable):
    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):
        return None

    def __init__(self):
        self.result = ""
        self.s_string = "HTTP_COOKIE"
        self.session_list = []
        self._commit = []
        
        return None

    def has(self, key=""):
        bool_v = False
        if self.out("SERVER_SOFTWARE") == AUTHOR:
            session_dict = self.x_get()
            if len(session_dict) > 0:
                if session_dict.get(key, "") != "":
                    bool_v = True
                elif session_dict.get(' {key}'.format(key=key), "") != "":
                    bool_v = True
                else:
                    bool_v = False
            else:
                bool_v = False
        else:

            if self.get(key) != "" and self.get(key) != None:

                bool_v = True
            else:
                bool_v = False

        return bool_v

    def set(self, key="", value="", duration=3600, url="", path="/"):

        if self.out("SERVER_SOFTWARE") == AUTHOR:
            session_string = str(key) + "=" + str(self._encode(value))
            self.set_x(session_string)
        else:
            self.set_k(key, self._encode(value), duration, url, path)

    def set_k(self, key="", value="", duration=3600, url="", path="/"):
        url_v = self.out("HTTP_HOST") + str(":") + str(self.out("SERVER_PORT", '')) if self.out(
            "HTTP_HOST") == "localhost" or self.out("HTTP_HOST") == "127.0.0.1" else self.out("HTTP_HOST")
        url = url if url != "" else url_v
        expires = datetime.datetime.utcnow(
        ) + datetime.timedelta(minutes=duration)  # minutes in 30 days
        cooKeys = cook.SimpleCookie(self.out(self.s_string))
        cooKeys[str(key)] = self._encode(value)
        cooKeys[str(key)]['domain'] = url
        cooKeys[str(key)]['path'] = '/'
        cooKeys[str(key)]['expires'] = expires.strftime(
            '%a, %d %b %Y %H:%M:%S')

        print(cooKeys[str(key)])

    def get(self, key=""):
        if self.out("SERVER_SOFTWARE") == AUTHOR:
            session_dict = self.x_get()

            if len(session_dict) > 0:
                if session_dict.get(key, "") != None or session_dict.get(key, "") != "":
                    try:
                        return ast.literal_eval(self._decode(session_dict.get(key, "")))
                    except Exception as err:
                        return self._decode(session_dict.get(key, ""))
                elif session_dict.get(' {key}'.format(key=key), "") != None or session_dict.get(
                        ' {key}'.format(key=key), "") != "":
                    try:
                        return ast.literal_eval(self._decode(session_dict.get(' {key}'.format(key=key), "")))
                    except Exception as err:
                        return self._decode(session_dict.get(' {key}'.format(key=key), ""))
                else:
                    return ""
            else:
                return ""
        else:
            cooKeys = cook.SimpleCookie()
            OsEnviron = self.out(self.s_string)

            if OsEnviron != None:
                cooKeys.load(OsEnviron)
                if key in cooKeys:

                    if cooKeys[key].value != None or cooKeys[key].value != "":

                        try:
                            return ast.literal_eval(self._decode(cooKeys[key].value))
                        except Exception as err:
                            return self._decode(cooKeys[key].value)

                    else:
                        return ""
                else:
                    return ""
            else:
                return ""

    def destroy(self, *args):
        bool_v = False
        if self.out("SERVER_SOFTWARE") == AUTHOR:

            session_result = self._delete(args)
            if session_result[0] == True:
                self._reset(session_result[1])
                bool_v = True
            else:
                bool_v = False

        else:
            cooKeys = cook.SimpleCookie(self.out(self.s_string))
            if PYVERSION_MA >= 3:

                cookv = cooKeys.items()
            else:
                cookv = cooKeys.iteritems()

            OsEnviron = self.out(self.s_string)

            bool_vx = False
            if self.s_string in os.environ:
                if args:

                    for key in args:

                        if self.get(key) != "":
                            duration = 3600 * 30 * 1000
                            self.set_k(key=key, duration=-duration)
                            bool_vx = True
                    bool_v = bool_vx

                else:

                    for key, v in cookv:

                        if self.get(key) != "":
                            duration = 3600 * 33 * 1000

                            self.set_k(key=key, duration=-duration)
                            bool_vx = True

                    bool_v = bool_vx

            else:
                bool_v = False
        return bool_v

    def set_x(self, session_string=""):
        if session_string != "":
            if self.s_string not in self.see():
                self.default(self.s_string, session_string)

            else:
                self.session_list.append(session_string)
                self._update(self.unqiue(self.session_list))
        else:
            return False

    def _update(self, session_string):
        dict_session = dict({self.s_string: session_string})
        try:
            self.update(dict_session)
            return True
        except Exception as e:
            return False

    def unqiue(self, session_list=[]):

        session_relist = []
        for l_session in session_list:
            session_string = str(l_session) + ";" + str(self.out(self.s_string, "")
                                                        ) if self.out(self.s_string, "") != "" else str(l_session)
            session_relist.append(session_string)

        initial_session = str(session_relist).replace("[' ", "").replace("]", "").replace(',', ";").replace("'",
                                                                                                            "").replace(
            "[", "")
        return self._unquie(initial_session)

    def _unquie(self, initial_session=""):
        try:
            unique_session = initial_session.split(";")
            re_unique = []
            [re_unique.append(x) for x in unique_session if x not in re_unique]

            return ";".join(re_unique)
        except Exception as err:
            return initial_session

    def _delete(self, session_key=tuple()):

        get_session = self.x_get()
        response = None
        if len(get_session) > 0:
            if len(session_key) > 0:

                try:
                    for k in session_key:
                        get_session.pop(k, None)
                    self._update(get_session)
                    response = True

                except Exception as err:
                    response = False
            else:
                try:
                    for k in dict(get_session):
                        get_session.pop(k, None)
                    response = True
                except Exception as err:
                    response = False
        else:
            response = False
        return response, get_session

    def _reset(self, session_dict=dict()):

        session_list = []
        if PYVERSION_MA >= 3:
            session_dict_l = session_dict.items()
        else:
            session_dict_l = session_dict.iteritems()

        for k, v in session_dict_l:
            session_list.append("{k}={v}".format(k=k, v=v))

        return self._update(";".join(session_list))

    def x_get(self):

        session_dict = {}
        session_string = ""
        try:
            session_string = self.out(self.s_string)
        except Exception as err:

            session_dict = {}
        try:
            for c_g in session_string.split(";"):
                k, v = c_g.split("=")
                session_dict.update({str(k): str(v)})
        except Exception as err:
            try:
                k, v = session_string.split("=")
                session_dict.update({str(k): str(v)})
            except Exception as err:
                session_dict = {}
        return session_dict

    def _encode(self, source):
        from pytonik.Functions.now import now
        from pytonik.Functions.rand import rand
        from pytonik.Hash import Hash
        sessionRAND =  Hash().hex_hash(rand().number(8), hex_type="{}".format(HASH_PRE["64"]), size=120)
        sessionTIMER = Hash().hex_hash(now().unix(), hex_type="{}".format(HASH_PRE["192"]), size=80)
        
        try:
            import six
            import base64
            if six.PY3:
                source = str(sessionPREFIX+sessionRAND+source+str(sessionTIMER)).encode('utf-8')
            content = base64.b64encode(source).decode('utf-8')
            return str(content)
        except Exception as err:
            return str(source)

    def _decode(self, source):
        
        try:
            import six
            import base64
            source = base64.b64decode(str(source)).decode()
            source = source.split("::")
            source = source[1][64:-192]
            try:
                return source if isinstance(int(source), int) == True else source
            except Exception as err:
                return Session()._decode(source)
        except Exception as err:
            return source



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 08/11/2019.

from pytonik.App import App



class Schema(App):

    def __getattr__(self, item):
        return item


    def __init__(self):
        return None

    def table(self, table):
        self.DB()
        drive = self.Driver
        if drive == "MYSQL":
            from pytonik.Driver.DB.MYSQL.Table import  Table
            return Table(table)

        if drive == "Oracle":
            from pytonik.Driver.DB.Oracle.Table import  Table
            return Table(table)

        if drive == "pyPgSQL":
            from pytonik.Driver.DB.pyPgSQL.Table import  Table
            return Table(table)

        if drive == "SQLite":
            from pytonik.Driver.DB.SQLite.Table import  Table
            return Table(table)

    def raw(self, string):
        return string

    def query(self, raw):
        return self.DB().query(raw)




###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###
import ast

class Config:


    def __init__(self, key=None, value=None):

        self.__key = key
        self.__value = value
        self.settings = {key : value}


    def add(self, key=None, value=None):

        try:
            if value is None:
                self.settings = key
            else:
                self.set(key, value)
        except Exception as err:
            print(err)

    def set(self, key=None, value=None):
        self.settings = {key : value}

    def get(self, key=None, empty=''):

        try:
            l = ast.literal_eval(self.settings)

            if str(key) in l:
                return l.get(key, empty)
            else:
                return ""
        except Exception as err:
            print(err)




# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 2/24/20.


class env:



    def __init__(self):
        return None

    def _e(self):

        from pytonik import Version, Config


        import sys, os

        if os.path.isdir(os.getcwd() + '/public'):
            host = os.getcwd()  # os.path.dirname(os.getcwd())

        else:
            host = os.path.dirname(os.getcwd())

        DS = str("/")

        envpath = host + DS + ".env"

        if os.path.isfile(envpath) == True:

            try:
                ##f = open(envpath, "r")
                return self._s(envpath)#f.read()
            except Exception as err:
                   print(err)
        else:
            print(".env file not found")


    def _s(self, env_file):

        f = open(env_file, "r")
        splt = f.read()
        env_vars = {}
        rd, rds = "", ""

        for ls in splt.split():

            if ls.startswith('#') or not ls.strip():
                continue
            try:


                k, v = ls.split("=")
                env_vars.update({k: v})
            except Exception as err:
                rd += ls

        brack_f = str(rd)[1:]
        brack_e = brack_f[:-1]
        rds += '{'+str(brack_e)+"," + str(env_vars)[1:][:-1] if len(env_vars) > 0 else '{'+brack_e
        rds += '}'

        return rds


# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 08/11/2019.

from pytonik.Editor import Template
from pytonik.Log import Log
from pytonik.App import App
import os
log_msg = Log()


class include(App):


    def __getattr__(self, item):

        return item

    def __call__(self, *args, **kwargs):

        return None

    def __init__(self, *args,  **kwargs):
        if len(args) > 0 or len(kwargs) > 0:
            if all(args) is not False:
                self.ex = self.include(*args,  **kwargs)
            else:
                self.ex = self.include(**kwargs)
        return None

    def __str__(self):

        return self.ex


    def include(self, path = "", is_temp=False):
        tempath = ""
        DS = str("/")
        if bool(is_temp) == True:
            tempath = DS+str(self.envrin('tempath'))
        if os.path.isdir(os.getcwd() + '/public'):
            host = str(os.getcwd()).replace("\\", "/")  # os.path.dirname(os.getcwd())

        else:
            host = str(os.path.dirname(os.getcwd())).replace("\\", "/")

       
        pth = path

        split = pth.split('.')


        fileExists = []
        for x in split:
            fileExists.append(x)

        try:

            dirF =  list(filter(None, fileExists))
            lf = ""
            for v in dirF:
                lf +="/"+v

            template_dir = host + DS + 'views' + str(tempath)+ str(lf)
            engine  = template_dir.rsplit('/', 1)[-1]
            direct = str(template_dir.rsplit('/', 1)[-2])
            
            if os.path.isdir(direct) == True:

                if 'views.html' not in template_dir+".html":

                    if os.path.isfile(template_dir+".html") == True:

                        loadm0 = self.loadmodule()
                        return self.read_html(direct, engine, loadm0)
                    else:
                      log_msg.error("The File '{filepath}' does not exists.".format(filepath=template_dir+".html"))
                      return "The File '{filepath}' does not exists.".format(filepath=template_dir+".html" )

            else:
              log_msg.error("The Folder '{filepath}' does not exists.".format(filepath=direct))
              return "The Folder '{filepath}' does not exists.".format(filepath=direct)

        except Exception as err:
                log_msg.error(err)
                return  err


# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 26/11/2019.

class pagination():

    def __getattr__(self, item):
        return item
    def __init__(self, *args, **kwargs):
        return None

    @staticmethod
    def number(total = 0, page = 0, url='/', title='Page', css=['', '', '']):
	
	
        content = '<ul class="{css}">'.format(css=css[0])


        if int(total) > 0:


            if (int(page) > 0) is True:

                content += '<li class="{css}"><a class="{css1}"  href="{url}/1" title="First {title}">&laquo;</a></li>'.format(title = title, url=url, css=css[1], css1=css[2])  # first link
                previous_link = 1 if abs(1 - page) == 0 else abs(1 - page)


                content += '<li class="{css}"><a  class="{css1}" href="{url}/{previous_link}" data-page="{previous_link}" title="Previous - {title}">&lt;</a></li>'.format(title = title,
                    url=url, previous_link=previous_link, css=css[1], css1=css[2])

                for c in range(1, int(total + 1)):

                    if (int(page) != int(c)) is True:

                        content += '<li  class="{css}"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url,
                                                                                                           c=str(c), css=css[1], css1=css[2])

                    elif (int(page) == int(c)) is True:

                        content += '<li  class="{css} active"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url, c=str(c), css=css[1], css1=css[2])


                    elif (int(page) == int(total)) is True:

                        content += '<li  class="{css} active"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url,
                                                                                                                 c=str(c), css=css[1], css1=css[2])

                    else:
                        content += '<li  class="{css}"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url,
                                                                                                                 c=str(c), css=css[1], css1=css[2])

            else:

                for c in range(1, int(total + 1)):
                    if (int(c) == int(1)) is True:

                        content += '<li  class="{css} active"><a  class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url,
                                                                                                                 c=str(c),  css=css[1], css1=css[2])


                    else:

                        content += '<li  class="{css}"><a class="{css1} href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(url=url,c=str(c), css=css[1], css1=css[2])

            if int(page) < int(total):
                next_link = int(page) + 1
                content += '<li class="{css}"><a class="{css1}" href="{url}/{next_link}" title="Next - {title}">&gt;</a></li>'.format(title = title, url=url,
                                                                                               next_link=next_link, css=css[1], css1=css[2])  # first link

                content += '<li  class="{css}" ><a  class="{css1}" href="{url}/{next_link}" data-page="{next_link}" title="Last - {title}">&raquo;</a></li>'. \
                    format(title = title, url=url, next_link=next_link, css=css[1], css1=css[2])
    
        content += '</ul>'
        if content is not None:
            return content
        else:
            return ""


    @staticmethod
    def alphabet(total = 0, page = '', url='/', title='Page', css=['', '', '']):
            
        alphabet_list = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
            
        content = '<ul class="{css}">'.format(css=css[0])
        page = page.upper().replace(' ', '')
        if int(total) > 0:

            for c in alphabet_list:
                if page != c:
                    content += '<li class="{css}"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url, c=str(c), css=css[1], css1=css[2])
                elif page == c:
                    content += '<li class="{css} active"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title = title, url=url, c=str(c), css=css[1], css1=css[2])


        content += '</ul>'
        if content is not None:
            return content
        else:
            return ""

    @staticmethod
    def alphabet_first_last(total=0, page='', title='Page', url='/', css=['', '', '']):

        alphabet_list = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
                         'T', 'U', 'V', 'W', 'X', 'Y', 'Z']

        content = '<ul class="{css}">'.format(css=css[0])
        page = page.upper().replace(' ', '')
        if int(total) > 0:

            content += '<li class="{css}"><a class="{css1}"  href="{url}/{page}" title="First - {title}">&laquo;</a></li>'.format(title=title,
                page=alphabet_list[0], url=url, css=css[1], css1=css[2])

            for c in alphabet_list:
                if page != c:
                    content += '<li class="{css}"><a class="{css1}" href="{url}/{c}" title="{title} - {c}">{c}</a></li>'.format(title=title,
                        url=url, c=str(c), css=css[1], css1=css[2])
                elif page == c:
                    content += '<li class="{css} active"><a class="{css1}" href="{url}/{c}" title="{title} {c}">{c}</a></li>'.format(title=title,
                        url=url, c=str(c), css=css[1], css1=css[2])

            content += '<li  class="{css}" ><a  class="{css1}" href="{url}/{next_link}" data-page="{next_link}" title="Last - {title}">&raquo;</a></li>'. \
                format(title=title, url=url, next_link=alphabet_list[-1], css=css[1], css1=css[2])

        content += '</ul>'
        if content is not None:
            return content
        else:
            return ""
    
    @staticmethod
    def next_previous(total = 0, page = 0, url='/', title='Page', css=['', '', '']):

         content = '<ul class="{css}">'.format(css=css[0])


         if int(total) > 0:


            if (int(page) > int(0)) is True:
                previous_link = 1 if (1 - page) == 0 else abs(1 - page)
                ac = [c for c in range(1, int(total + 1)) if c == page]

                tl = total if ac != [page]  else previous_link

                if ac != [page]:
                    active  = 'active'
                elif ((page - total) == 0) is True:
                    active = 'active'
                else:
                    active = ''

                content += '<li class="{css} {active}"><a  class="{css1}" href="{url}/{previous_link}" data-page="{previous_link}" title="Previous - {title}">Previous</a></li>'.format(title = title,
                    url=url, active = active, previous_link=tl, css=css[1], css1=css[2])

            if (int(page) < int(total)) is True:
                next_link = int(page) + 1
                ac = [c for c in range(1, int(total + 1)) if c == page ]

                active = 'active' if ac == [page] else ""
                content += '<li  class="{css} {active}" ><a  class="{css1}" href="{url}/{next_link}" data-page="{next_link}" title="Next - {title}">Next</a></li>'. \
                        format(title = title, active=active, url=url, next_link=next_link, css=css[1], css1=css[2])
                        
         content += '</ul>'

         if content is not None:
            return content
         else:
            return ""

    @staticmethod
    def first_last(total=0, page=0, url='/', title = 'Page', css=['', '', '']):

        content = '<ul class="{css}">'.format(css=css[0])

        if int(total) > 0:

            if (int(page) > int(0)) is True:


                ac = [c for c in range(1, int(total + 1)) if c == page]


                if ac != [page]:
                    active = 'active'
                elif ((page - total) == 0) is True:
                    active = 'active'
                else:
                    active = ''

                content += '<li class="{css}"><a class="{css1}"  href="{url}/1" title="First - {title}">&laquo;</a></li>'.format(title = title,
                    url=url, css=css[1], css1=css[2])  # first link

                previous_link = 1 if (1 - page) == 0 else abs(1 - page)

                tl = total if ac != [page]  else previous_link

                content += '<li class="{css} {active}"><a  class="{css1}" href="{url}/{previous_link}" data-page="{previous_link}" title="Previous - {title}">Previous</a></li>'.format(title=title, url=url, active=active, previous_link=tl, css=css[1], css1=css[2])

            if (int(page) < int(total)) is True:
                next_link = int(page) + 1
                ac = [c for c in range(1, int(total + 1)) if c == page]

                active = 'active' if ac == [page] else ""
                content += '<li  class="{css} {active}" ><a  class="{css1}" href="{url}/{next_link}" data-page="{next_link}" title="Next - {title}">Next</a></li>'. \
                    format(title = title, active=active, url=url, next_link=next_link, css=css[1], css1=css[2])

                content += '<li  class="{css}" ><a  class="{css1}" href="{url}/{next_link}" data-page="{next_link}" title="Last - {title}">&raquo;</a></li>'. \
                    format(title = title, url=url, next_link=total, css=css[1], css1=css[2])

        content += '</ul>'

        if content is not None:
            return content
        else:
            return ""


from setuptools import setup, find_namespace_packages

from pytonik import Version

with open("README.md", "r") as fd:
    longdescription = fd.read()


setup(
    name='pytonik',
    version=Version.VERSION_TEXT + Version.EDITION,
    description='Pytonik is a python framework built to enhance web development fast and easy, also help web developers to build more apps with less codes',
    url="https://pytonik.readthedocs.io/en/latest",
    author='pytonik',
    author_email='info@pytonik.com',
    maintainer='Emmanuel Essien , Uduak Essien , Claret Nnamocha ',
    maintainer_email='emmamartinscm@gmail.com , acushla4real@gmail.com , devclareo@gmail.com',
    include_package_data=True,
    packages=find_namespace_packages(include=['*', '']),
    long_description=longdescription,
    long_description_content_type='text/markdown',
    license=Version.LICENSE,
    keywords=Version.KEYWORDS,
    entry_points={
        'console_scripts': ['pytonik = pytonik.cmd.help:main', 'pytonik-install = pytonik.cmd.install:main', 'pytonik-start = pytonik.cmd.start:main', 'pytonik-docs = pytonik.cmd.doc:main',  'pytonik-server = pytonik.cmd.server:main']
    },
    install_requires=['Pillow', 'colorama',
                      'mysql-connector-python', 'psycopg2-binary', 'cx-Oracle'],
    zip_safe=False,
    classifiers=[
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Office/Business',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python'
    ],
    python_requires='>=2.7, >=2.7.*, >=3.*',
)



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import sys

VERSION = (1, 10, 0, 'b', 1)

if VERSION[3] and VERSION[4]:
    VERSION_TEXT = '{0}.{1}.{2}{3}{4}'.format(*VERSION)
else:
    VERSION_TEXT = '{0}.{1}.{2}'.format(*VERSION[0:3])

VERSION_EXTRA = ''
LICENSE = 'GPL3'
EDITION = ''  # Added in package names, after the version
KEYWORDS = "mvc, oop, module, python, framework, web, app, pytonik, web, web development"

PYVERSION_MA = sys.version_info.major
PYVERSION_MI = sys.version_info.minor

AUTHOR = "Pytonik"
ORG = "Betacodings"

sessionPREFIX = "PYTONIKSESSID::"

MIME_TYPES = [
    {'ext': '.ai', 'type': 'application/postscript', 'mode': 'rb'},
    {'ext': '.aif', 'type': 'audio/x-aiff', 'mode': 'rb'},
    {'ext': '.aifc', 'type': 'audio/x-aiff', 'mode': 'rb'},
    {'ext': '.aiff', 'type': 'audio/x-aiff', 'mode': 'rb'},
    {'ext': '.asc', 'type': 'text/plain', 'mode': 'r'},
    {'ext': '.atom', 'type': 'application/atom+xml', 'mode': 'rb'},
    {'ext': '.au', 'type': 'audio/basic', 'mode': 'rb'},
    {'ext': '.avi', 'type': 'video/x-msvideo', 'mode': 'rb'},
    {'ext': '.bcpio', 'type': 'application/x-bcpio', 'mode': 'rb'},
    {'ext': '.bin', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.bmp', 'type': 'image/bmp', 'mode': 'rb'},
    {'ext': '.cdf', 'type': 'application/x-netcdf', 'mode': 'rb'},
    {'ext': '.cgm', 'type': 'image/cgm', 'mode': 'rb'},
    {'ext': '.class', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.cpio', 'type': 'application/x-cpio', 'mode': 'rb'},
    {'ext': '.cpt', 'type': 'application/mac-compactpro', 'mode': 'rb'},
    {'ext': '.csh', 'type': 'application/x-csh', 'mode': 'rb'},
    {'ext': '.css', 'type': 'text/css', 'mode': 'r'},
    {'ext': '.dcr', 'type': 'application/x-director', 'mode': 'rb'},
    {'ext': '.dif', 'type': 'video/x-dv', 'mode': 'rb'},
    {'ext': '.dir', 'type': 'application/x-director', 'mode': 'rb'},
    {'ext': '.djv', 'type': 'image/vnd.djvu', 'mode': 'rb'},
    {'ext': '.djvu', 'type': 'image/vnd.djvu', 'mode': 'rb'},
    {'ext': '.dll', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.dmg', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.dms', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.doc', 'type': 'application/msword', 'mode': 'rb'},
    {'ext': '.dtd', 'type': 'application/xml-dtd', 'mode': 'rb'},
    {'ext': '.dv', 'type': 'video/x-dv', 'mode': 'rb'},
    {'ext': '.dvi', 'type': 'application/x-dvi', 'mode': 'rb'},
    {'ext': '.dxr', 'type': 'application/x-director', 'mode': 'rb'},
    {'ext': '.eps', 'type': 'application/postscript', 'mode': 'rb'},
    {'ext': '.etx', 'type': 'text/x-setext', 'mode': 'r'},
    {'ext': '.exe', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.ez', 'type': 'application/andrew-inset', 'mode': 'rb'},
    {'ext': '.gif', 'type': 'image/gif', 'mode': 'rb'},
    {'ext': '.gram', 'type': 'application/srgs', 'mode': 'rb'},
    {'ext': '.grxml', 'type': 'application/srgs+xml', 'mode': 'rb'},
    {'ext': '.gtar', 'type': 'application/x-gtar', 'mode': 'rb'},
    {'ext': '.hdf', 'type': 'application/x-hdf', 'mode': 'rb'},
    {'ext': '.hqx', 'type': 'application/mac-binhex40', 'mode': 'rb'},
    {'ext': '.htm', 'type': 'text/html', 'mode': 'r'},
    {'ext': '.html', 'type': 'text/html', 'mode': 'r'},
    {'ext': '.ice', 'type': 'x-conference/x-cooltalk', 'mode': 'rb'},
    {'ext': '.ico', 'type': 'image/x-icon', 'mode': 'rb'},
    {'ext': '.ics', 'type': 'text/calendar', 'mode': 'r'},
    {'ext': '.ief', 'type': 'image/ief', 'mode': 'rb'},
    {'ext': '.ifb', 'type': 'text/calendar', 'mode': 'r'},
    {'ext': '.iges', 'type': 'model/iges', 'mode': 'rb'},
    {'ext': '.igs', 'type': 'model/iges', 'mode': 'rb'},
    {'ext': '.jnlp', 'type': 'application/x-java-jnlp-file', 'mode': 'rb'},
    {'ext': '.jp2', 'type': 'image/jp2', 'mode': 'rb'},
    {'ext': '.jpe', 'type': 'image/jpeg', 'mode': 'rb'},
    {'ext': '.jpeg', 'type': 'image/jpeg', 'mode': 'rb'},
    {'ext': '.jpg', 'type': 'image/jpeg', 'mode': 'rb'},
    {'ext': '.kar', 'type': 'audio/midi', 'mode': 'rb'},
    {'ext': '.latex', 'type': 'application/x-latex', 'mode': 'rb'},
    {'ext': '.lha', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.lzh', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.m3u', 'type': 'audio/x-mpegurl', 'mode': 'rb'},
    {'ext': '.m4a', 'type': 'audio/mp4a-latm', 'mode': 'rb'},
    {'ext': '.m4b', 'type': 'audio/mp4a-latm', 'mode': 'rb'},
    {'ext': '.m4p', 'type': 'audio/mp4a-latm', 'mode': 'rb'},
    {'ext': '.m4u', 'type': 'video/vnd.mpegurl', 'mode': 'rb'},
    {'ext': '.m4v', 'type': 'video/x-m4v', 'mode': 'rb'},
    {'ext': '.mac', 'type': 'image/x-macpaint', 'mode': 'rb'},
    {'ext': '.man', 'type': 'application/x-troff-man', 'mode': 'rb'},
    {'ext': '.mathml', 'type': 'application/mathml+xml', 'mode': 'rb'},
    {'ext': '.me', 'type': 'application/x-troff-me', 'mode': 'rb'},
    {'ext': '.mesh', 'type': 'model/mesh', 'mode': 'rb'},
    {'ext': '.mid', 'type': 'audio/midi', 'mode': 'rb'},
    {'ext': '.midi', 'type': 'audio/midi', 'mode': 'rb'},
    {'ext': '.mif', 'type': 'application/vnd.mif', 'mode': 'rb'},
    {'ext': '.mov', 'type': 'video/quicktime', 'mode': 'rb'},
    {'ext': '.movie', 'type': 'video/x-sgi-movie', 'mode': 'rb'},
    {'ext': '.mp2', 'type': 'audio/mpeg', 'mode': 'rb'},
    {'ext': '.mp3', 'type': 'audio/mpeg', 'mode': 'rb'},
    {'ext': '.mp4', 'type': 'video/mp4', 'mode': 'rb'},
    {'ext': '.mpe', 'type': 'video/mpeg', 'mode': 'rb'},
    {'ext': '.mpeg', 'type': 'video/mpeg', 'mode': 'rb'},
    {'ext': '.mpg', 'type': 'video/mpeg', 'mode': 'rb'},
    {'ext': '.mpga', 'type': 'audio/mpeg', 'mode': 'rb'},
    {'ext': '.ms', 'type': 'application/x-troff-ms', 'mode': 'rb'},
    {'ext': '.msh', 'type': 'model/mesh', 'mode': 'rb'},
    {'ext': '.mxu', 'type': 'video/vnd.mpegurl', 'mode': 'rb'},
    {'ext': '.nc', 'type': 'application/x-netcdf', 'mode': 'rb'},
    {'ext': '.oda', 'type': 'application/oda', 'mode': 'rb'},
    {'ext': '.ogg', 'type': 'application/ogg', 'mode': 'rb'},
    {'ext': '.pbm', 'type': 'image/x-portable-bitmap', 'mode': 'rb'},
    {'ext': '.pct', 'type': 'image/pict', 'mode': 'rb'},
    {'ext': '.pdb', 'type': 'chemical/x-pdb', 'mode': 'rb'},
    {'ext': '.pdf', 'type': 'application/pdf', 'mode': 'rb'},
    {'ext': '.pgm', 'type': 'image/x-portable-graymap', 'mode': 'rb'},
    {'ext': '.pgn', 'type': 'application/x-chess-pgn', 'mode': 'rb'},
    {'ext': '.pic', 'type': 'image/pict', 'mode': 'rb'},
    {'ext': '.pict', 'type': 'image/pict', 'mode': 'rb'},
    {'ext': '.png', 'type': 'image/png', 'mode': 'rb'},
    {'ext': '.pnm', 'type': 'image/x-portable-anymap', 'mode': 'rb'},
    {'ext': '.pnt', 'type': 'image/x-macpaint', 'mode': 'rb'},
    {'ext': '.pntg', 'type': 'image/x-macpaint', 'mode': 'rb'},
    {'ext': '.ppm', 'type': 'image/x-portable-pixmap', 'mode': 'rb'},
    {'ext': '.ppt', 'type': 'application/vnd.ms-powerpoint', 'mode': 'rb'},
    {'ext': '.ps', 'type': 'application/postscript', 'mode': 'rb'},
    {'ext': '.qt', 'type': 'video/quicktime', 'mode': 'rb'},
    {'ext': '.qti', 'type': 'image/x-quicktime', 'mode': 'rb'},
    {'ext': '.qtif', 'type': 'image/x-quicktime', 'mode': 'rb'},
    {'ext': '.ra', 'type': 'audio/x-pn-realaudio', 'mode': 'rb'},
    {'ext': '.ram', 'type': 'audio/x-pn-realaudio', 'mode': 'rb'},
    {'ext': '.ras', 'type': 'image/x-cmu-raster', 'mode': 'rb'},
    {'ext': '.rdf', 'type': 'application/rdf+xml', 'mode': 'rb'},
    {'ext': '.rgb', 'type': 'image/x-rgb', 'mode': 'rb'},
    {'ext': '.rm', 'type': 'application/vnd.rn-realmedia', 'mode': 'rb'},
    {'ext': '.roff', 'type': 'application/x-troff', 'mode': 'rb'},
    {'ext': '.rtf', 'type': 'text/rtf', 'mode': 'r'},
    {'ext': '.rtx', 'type': 'text/richtext', 'mode': 'r'},
    {'ext': '.sgm', 'type': 'text/sgml', 'mode': 'r'},
    {'ext': '.sgml', 'type': 'text/sgml', 'mode': 'r'},
    {'ext': '.sh', 'type': 'application/x-sh', 'mode': 'rb'},
    {'ext': '.shar', 'type': 'application/x-shar', 'mode': 'rb'},
    {'ext': '.silo', 'type': 'model/mesh', 'mode': 'rb'},
    {'ext': '.sit', 'type': 'application/x-stuffit', 'mode': 'rb'},
    {'ext': '.skd', 'type': 'application/x-koan', 'mode': 'rb'},
    {'ext': '.skm', 'type': 'application/x-koan', 'mode': 'rb'},
    {'ext': '.skp', 'type': 'application/x-koan', 'mode': 'rb'},
    {'ext': '.skt', 'type': 'application/x-koan', 'mode': 'rb'},
    {'ext': '.smi', 'type': 'application/smil', 'mode': 'rb'},
    {'ext': '.smil', 'type': 'application/smil', 'mode': 'rb'},
    {'ext': '.snd', 'type': 'audio/basic', 'mode': 'rb'},
    {'ext': '.so', 'type': 'application/octet-stream', 'mode': 'rb'},
    {'ext': '.spl', 'type': 'application/x-futuresplash', 'mode': 'rb'},
    {'ext': '.src', 'type': 'application/x-wais-source', 'mode': 'rb'},
    {'ext': '.sv4cpio', 'type': 'application/x-sv4cpio', 'mode': 'rb'},
    {'ext': '.sv4crc', 'type': 'application/x-sv4crc', 'mode': 'rb'},
    {'ext': '.svg', 'type': 'image/svg+xml', 'mode': 'rb'},
    {'ext': '.swf', 'type': 'application/x-shockwave-flash', 'mode': 'rb'},
    {'ext': '.t', 'type': 'application/x-troff', 'mode': 'rb'},
    {'ext': '.tar', 'type': 'application/x-tar', 'mode': 'rb'},
    {'ext': '.tcl', 'type': 'application/x-tcl', 'mode': 'rb'},
    {'ext': '.tex', 'type': 'application/x-tex', 'mode': 'rb'},
    {'ext': '.texi', 'type': 'application/x-texinfo', 'mode': 'rb'},
    {'ext': '.texinfo', 'type': 'application/x-texinfo', 'mode': 'rb'},
    {'ext': '.tif', 'type': 'image/tiff', 'mode': 'rb'},
    {'ext': '.tiff', 'type': 'image/tiff', 'mode': 'rb'},
    {'ext': '.tr', 'type': 'application/x-troff', 'mode': 'rb'},
    {'ext': '.tsv', 'type': 'text/tab-separated-values', 'mode': 'r'},
    {'ext': '.txt', 'type': 'text/plain', 'mode': 'r'},
    {'ext': '.ustar', 'type': 'application/x-ustar', 'mode': 'rb'},
    {'ext': '.vcd', 'type': 'application/x-cdlink', 'mode': 'rb'},
    {'ext': '.vrml', 'type': 'model/vrml', 'mode': 'rb'},
    {'ext': '.vxml', 'type': 'application/voicexml+xml', 'mode': 'rb'},
    {'ext': '.wav', 'type': 'audio/x-wav', 'mode': 'rb'},
    {'ext': '.wbmp', 'type': 'image/vnd.wap.wbmp', 'mode': 'rb'},
    {'ext': '.wbmxl', 'type': 'application/vnd.wap.wbxml', 'mode': 'rb'},
    {'ext': '.wml', 'type': 'text/vnd.wap.wml', 'mode': 'r'},
    {'ext': '.wmlc', 'type': 'application/vnd.wap.wmlc', 'mode': 'rb'},
    {'ext': '.wmls', 'type': 'text/vnd.wap.wmlscript', 'mode': 'r'},
    {'ext': '.wmlsc', 'type': 'application/vnd.wap.wmlscriptc', 'mode': 'rb'},
    {'ext': '.wrl', 'type': 'model/vrml', 'mode': 'rb'},
    {'ext': '.xbm', 'type': 'image/x-xbitmap', 'mode': 'rb'},
    {'ext': '.xht', 'type': 'application/xhtml+xml', 'mode': 'rb'},
    {'ext': '.xhtml', 'type': 'application/xhtml+xml', 'mode': 'rb'},
    {'ext': '.xls', 'type': 'application/vnd.ms-excel', 'mode': 'rb'},
    {'ext': '.xml', 'type': 'application/xml', 'mode': 'rb'},
    {'ext': '.xpm', 'type': 'image/x-xpixmap', 'mode': 'rb'},
    {'ext': '.xsl', 'type': 'application/xml', 'mode': 'rb'},
    {'ext': '.xslt', 'type': 'application/xslt+xml', 'mode': 'rb'},
    {'ext': '.xul', 'type': 'application/vnd.mozilla.xu+xml', 'mode': 'rb'},
    {'ext': '.xwd', 'type': 'image/x-xwindowdump', 'mode': 'rb'},
    {'ext': '.xyz', 'type': 'chemical/x-xyz', 'mode': 'rb'},
    {'ext': '.zip', 'type': 'application/zip', 'mode': 'rb'},
    {'ext': '.jpg', 'type': 'image/jpg', 'mode': 'rb'},
    {'ext': '.mp4', 'type': 'audio/mp4', 'mode': 'rb'},
    {'ext': '.js', 'type': 'application/javascript', 'mode': 'r'},
    {'ext': '.ttf', 'type': 'application/x-font-ttf', 'mode': 'rb'},
    {'ext': '.woff2', 'type': 'application/x-font-woff2', 'mode': 'rb'},
    {'ext': '.woff', 'type': 'application/x-font-woff', 'mode': 'r'},
    {'ext': '.wav', 'type': 'audio/wav', 'mode': 'rb'},
    {'ext': '.mpe', 'type': 'audio/mpeg', 'mode': 'rb'},
    {'ext': '.mpeg', 'type': 'audio/mpeg', 'mode': 'rb'},
    {'ext': '.json', 'type': 'application/json', 'mode': 'rb'}
]


HTTP_CODE = {
    '100': 'Continue',
    '101': 'Switching Protocols',
    '102': 'Processing',
    '103': 'Processing',
    '122': 'Request-uri too long',
    '200': 'Ok',
    '201': 'Created',
    '202': 'Accepted',
    '203': 'Partial Information',
    '204': 'No Response',
    '205': 'Reset Content',
    '206': 'Partial Content',
    '207': 'Multi-Status',
    '208': 'Already Reported',
    '226': 'Im Used',
    '301': 'Moved',
    '302': 'Found',
    '303': 'Method',
    '304': 'Not Modified',
    '306': 'Switch Proxy',
    '307': 'Temporary Redirect',
    '308': 'Permanent Redirect',
    '400': 'Bad Request',
    '401': 'Unauthorized',
    '402': 'Payment Required',
    '403': 'Forbidden',
    '404': 'Not Found',
    '405': 'Method Not Allowed',
    '406': 'Not Acceptable',
    '407': 'Proxy Authentication Required',
    '408': 'Request Timeout',
    '409': 'Conflict',
    '410': 'Gone',
    '411': 'Length Required',
    '412': 'Precondition Failed',
    '413': 'Request Entity Too Large',
    '414': 'Request URI Too Large',
    '415': 'Unsupported Media Type',
    '416': 'Requested Rage Not Satisfiable',
    '417': 'Expectation Failed',
    '418': 'I\'m a teapot',
    '420': 'Enhance Your Calm',
    '421': 'Misdirected Request',
    '422': 'Unprocessable Entity',
    '423': 'Locked',
    '424': 'Failed Dependency',
    '426': 'Upgrade Required',
    '428': 'Precondition Required',
    '429': 'Too Many Requests',
    '431': 'Request Header Fields Too Large',
    '444': 'No Response',
    '449': 'Retry With',
    '450': 'Blocked By Windows Parental Controls',
    '451': 'Wrong Exchange Server\'s mailbox',
    '499': 'Client Closed Request',
    '500': 'Internal Error',
    '501': 'Not Implemented',
    '502': 'Service temporarily overloaded',
    '503': 'Gateway timeout',
    '504': 'Gateway Timeout',
    '505': 'Http Version Not Supported',
    '506': 'Variant Also Negotiates',
    '507': 'Insufficient Storage',
    '508': 'Loop Detected',
    '509': 'Bandwidth Limit Exceeded',
    '510': 'Not Extended',
    '511': 'Network Authentication Required',
    '598': 'Network Read Timeout Error',
    '599': 'Network Connect Timeout Error',
}

HASH_PRE = {
            "128": "sha256",
            "192": "sha384",
            "256": "sha512",
            "56" : "sha224",
            "80" : "sha1",
            "64" : "md5",
}



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###
from pytonik import Version
from pytonik.Log import Log
import socket
import json

log_msg = Log()

try:
    import http.client as htp
    from  http import client
    import urllib as urllib
except Exception as err:
    import httplib as htp
    import urllib2 as urllib


class curl:
    def __getattr__(self, item):

        return item

    def __init__(self):

        self.__var()
        self.URL = "URL"
        self.HTTPHEADER = "HTTPHEADER"
        self.TIMEOUT = "TIMEOUT"
        self.FOLLOWLOCATION = "FOLLOWLOCATION"
        self.CONTENTHEADER = "CONTENTHEADER"
        self.POSTFIELDS = "POSTFIELDS"
        self.ACCEPTHEADER = "ACCEPTHEADER"
        self.POST = 'POST'
        self.GET = 'GET'
        self.HEAD = 'HEAD'
        self.PUT = 'PUT'
        self.PORT = "PORT"

        return

    def __var(self):
        self.h = dict()
        self.res = ""
        self.method = ""
        self.params = ""
        self.link = ""
        self.status = ""
        self.reason = ""
        self.path = ""
        self.source_address = None
        self.blocksize = 8192
        self.body = None
        self.conn = ""
        self.http = ""
        self.output = ""
        self.pt = None
        self.tout = socket._GLOBAL_DEFAULT_TIMEOUT


    def set(self, options, actions=""):

        if options == self.POST:
            self.__method(self.POST)
            if actions == 1:
                self.path = ""
            else:
                self.__path(actions)
        if options == self.POSTFIELDS:
            self.__body(actions)

        if options == self.GET:
            self.__method(self.GET)
            if options != "":
                self.__path(actions)

        if options == self.HEAD:
            self.__method(self.HEAD)
            if actions != "":
                self.__path(actions)

        if options == self.PUT:
            self.__method(self.PUT)
            if actions != "":
                self.__path(actions)

        if options == self.URL:
            self.__url(actions)

        if options == self.PORT:
            self.__port(actions)

        if options == self.CONTENTHEADER:
            self.__contentType(actions)

        if options == self.ACCEPTHEADER:
            self.__accept(actions)

        if options == self.HEADER:
            self.__header(actions)

        if options == self.TIMEOUT:
            self.__timeout(actions)

        return self

    def finish(self):

        try:

            if self.method == self.GET or self.method == self.HEAD:
                if 's' in self.http:
                    self.conn = htp.HTTPSConnection(host=self.link, port=self.pt, timeout=self.tout,
                                                    source_address=self.source_address, blocksize=self.blocksize)
                else:
                    self.conn = htp.HTTPConnection(host=self.link, port=self.pt, timeout=self.tout,
                                                   source_address=self.source_address, blocksize=self.blocksize)
            else:
                if 's' in self.http:
                    self.conn = htp.HTTPSConnection(host=self.link, port=self.pt, timeout=self.tout,
                                                    source_address=self.source_address, blocksize=self.blocksize)
                else:
                    self.conn = htp.HTTPConnection(host=self.link, port=self.pt, timeout=self.tout,
                                                   source_address=self.source_address, blocksize=self.blocksize)

            self.conn.request(self.method, self.path, self.body, self.h)
            self.res = self.conn.getresponse()
            self.response()
            self.conn.close()
        except Exception as err:
            log_msg.error(err)

        return self

    def __url(self, url=""):
        lh = ['ftps://', 'ftp://', "https://", "http://"]
        for ht in lh:
            if ht in url:
                self.http = ht
                self.link = url.replace(ht, "")
        return self

    def __body(self, body=""):
        self.body = urllib.parse.urlencode(body)
        return self

    def __method(self, method):
        self.method = method
        return self

    def __accept(self, accept):
        header = {'Accept': accept}
        return self.array_dict(header)


    def __contentType(self, contT):

        header = {'Content-type': contT}
        return self.array_dict(header)


    def __header(self, header):
        return self.array_dict(header)


    def array_dict(self, ldict):
        ldict = ldict
        return self.h.update(ldict)

    def __port(self, pt):
        self.pt = pt
        return self

    def __path(self, path=""):
        self.path = "/" if path  == "" else path
        return self

    def __timeout(self, tout):
        self.tout = tout
        return self

    def response(self):
        self.status = self.res.status
        self.reason = self.res.reason
        self.output = self.res.read()
        return self

    def result(self, decode=""):
        if decode != "":
            reSlt = self.output.decode(decode)
        else:
            reSlt = self.output

        return reSlt

    def open(self, URL):
        try:
            import urllib2
            self.output =  urllib2.urlopen(URL).read()
            return self
        except Exception as err:
            import urllib.request
            self.output =  urllib.request.urlopen(URL).read()
            return self
    
    def request(self, URL, data):
        
        try:
            import urllib2
            data_v = urllib2.Request(URL, data)
            self.output =  urllib2.urlopen(URL).read()
            return self
        except Exception as err:
            import urllib.request
            data_v = urllib.parse.urlencode(data)
            data_v = data_v.encode('utf-8') # data should be bytes
            req = urllib.request.Request(URL, data_v)
            self.output =  urllib.request.urlopen(URL).read()
            return self
